#ifndef _PARSER_H_
#define _PARSER_H_

#include "event/event.h"
#include "event/listeners.h"
#include "unit/unit.h"
#include "msg/msg.h"
#include <pthread.h>


#define HTTP_STR "http://"

struct listener;
struct unit;


typedef struct parser* (*FUNC_PARSER_FACTORY)(struct unit*);

typedef struct parser {
	struct unit* unit;
	
	void* (*func_parse)(void*);
	
	// parser is an event publisher
	struct listener_list_entry* evt_listeners;
	void (*func_publish_event)(struct event*, struct parser*);
	
	// parser is a msg subscriber 
	void (*func_msg_received)(struct msg*, struct listener*);

	FUNC_PARSER_FACTORY factory;
	
} PARSER;
typedef PARSER* PARSER_PTR;

typedef struct parsing_thread_data {
	MSG_PTR msg;
	struct parser* parser;
} PARSINGTHREADDATA;
typedef PARSINGTHREADDATA* PARSINGTHREADDATA_PTR;

// parser is a msg subscriber 
void parser_msg_received(MSG_PTR msg, struct listener* node);
//void parser_event_received(EVENT_PTR evt, struct listener* node);



// parser is an event publisher
int parser_add_evtlistener(PARSER_PTR parser, struct listener* evt_list);
int parser_add_evtlistener_unit(PARSER_PTR parser, struct unit* u);
void parser_publish_event(EVENT_PTR evt, struct parser* parser);


PARSER_PTR parser_create(struct unit* u);
PARSER_PTR slp_parser_create(struct unit* u);
PARSER_PTR wsd_parser_create(struct unit* u);
PARSER_PTR wsdl_parser_create(struct unit* u);
PARSER_PTR ssdp_parser_create(struct unit* u);
PARSER_PTR upnp_devicedescr_parser_create(struct unit* u);
PARSER_PTR upnp_servicedescr_parser_create(struct unit* u);
PARSER_PTR soap_parser_create(struct unit* u);
PARSER_PTR jrmp_parser_create(struct unit* u);


void slp_parse_msg(PARSER_PTR parser, MSG_PTR message /*char* msg, int len*/, EVENTSRC_PTR evt_src /*, pthread_t thread_event_src*/);
void wsd_parse_msg(PARSER_PTR parser, MSG_PTR message , EVENTSRC_PTR evt_src);
void *slp_threaded_msg_parsing(/*PARSINGTHREADDATA_PTR data* */ void*);
void *wsd_threaded_msg_parsing(void* ptr);

PARSER_PTR upnp_parser_create(struct unit* u);
#endif //_PARSER_H_

