#ifndef _WSDLMSG_H_

#define _WSDLMSG_H_ 
typedef struct 
{
	char *name;
	char *type;
	char *element;
	struct PART* next;
}PART;
typedef struct MESSAGE 
{
	char *name;
	struct PART* parts;
	struct MESSAGE* next;
}MESSAGE_STR;
typedef struct
{
	char *name;
	char *message;
}OPERATION_ELEMENT;
typedef struct 
{
	char *name;
	struct OPERATION_ELEMENT* inputs;	
	struct OPERATION_ELEMENT* outputs;
	struct OPERATION_ELEMENT* faults;
	struct OPERATION* next;
}OPERATION;
typedef struct 
{
	char *name;
	struct OPERATION* operations;	
}PORT_TYPE;

typedef struct 
{
	char *location;
}ADDRESS;

typedef struct 
{
	char *name;
	char *binding;	
	struct ADDRESS* address;		
}PORT;

typedef struct 
{
	char *name;
	struct PORT* port;	
}SERVICE;

typedef struct 
{
	char *style;
	char *transport;	
}SOAPBINDING;

typedef struct 
{
	char *soapAction;
	char *style;
}SOAP_OP;

typedef struct 
{
	char *encodingStyle;
	char *nameSpace;
	char *use;
}SOAP_BODY;

typedef struct 
{
	struct SOAP_BODY* soapBody;
}BIN_OP_INPUT;

typedef struct 
{
	struct SOAP_BODY* soapBody;
}BIN_OP_OUTPUT;

typedef struct 
{
	char *name;
	struct SOAP_OP* soapOp;
	struct BIN_OP_INPUT* binOpInput;
	struct BIN_OP_OUTPUT* binOpOutput;
	struct BINDING_OP* next;
}BINDING_OP;

typedef struct 
{
	char *name;
	char *type;
	struct SOAPBINDING* soapBinding;
	struct BINDING_OP* binding_Op;	
}BINDING;

typedef struct
{
	char *minOccurs;
	char *maxOccurs;
	char *name;
	char *type;
	struct ELEMENT_PART* next;
}ELEMENT_PART;

typedef struct 
{
	struct ELEMENT_PART* elements;
}SEQUENCE;
typedef struct
{
	struct SEQUENCE* sequence;
}SIMPLE_TYPE;

typedef struct 
{
	struct SEQUENCE* sequence;
}COMPLEX_TYPE;

typedef struct 
{
	char *name;
	struct SIMPLE_TYPE* simpletypes;
	struct COMPLEX_TYPES* complextypes;
	struct ELEMENT_HEAD* next;
}ELEMENT_HEAD;
typedef struct
{
	char *elementFormDefault;
	char *targetNameSpace;
	struct ELEMENT_HEAD* elements;
}SCHEMA;
typedef struct 
{
	struct SCHEMA* schema;
}TYPES;

typedef struct 
{
	struct TYPES* types;
	struct MESSAGE* messages;	
	struct PORT_TYPE* port_types;
	struct SERVICE* service;
	struct BINDING* binding;
}WSDLMSG;

typedef WSDLMSG* WSDLMSG_PTR;

WSDLMSG_PTR wsdlmsg_create();

void wsdl_PrintWSDLMSG(WSDLMSG_PTR);

int wsdlmsg_set_message(WSDLMSG_PTR msg,MESSAGE_STR *messages);
int wsdlmsg_set_porttype(WSDLMSG_PTR msg,PORT_TYPE *port_types);
int wsdlmsg_set_service(WSDLMSG_PTR msg, SERVICE *service);
int wsdlmsg_set_binding(WSDLMSG_PTR msg,BINDING *binding);
int wsdlmsg_set_binding_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_binding_type(WSDLMSG_PTR msg,char *type);
int wsdlmsg_set_binding_soapbinding(WSDLMSG_PTR msg, SOAPBINDING *soapBinding);
int wsdlmsg_set_binding_bindingop(WSDLMSG_PTR msg,BINDING_OP *binding_Op);
int wsdlmsg_set_binding_bindingop_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_binding_bindingop_soapop(WSDLMSG_PTR msg,SOAP_OP *soapOp);
int wsdlmsg_set_binding_bindingop_binopinput(WSDLMSG_PTR msg,BIN_OP_INPUT *binOpInput);
int wsdlmsg_set_binding_bindingop_binopoutput(WSDLMSG_PTR msg,BIN_OP_OUTPUT *binOpOutput);
int wsdlmsg_set_binding_bindingop_binopinput_soapbody(WSDLMSG_PTR msg,SOAP_BODY *soapBody);
int wsdlmsg_set_binding_bindingop_binopoutput_soapbody(WSDLMSG_PTR msg,SOAP_BODY *soapBody);
int wsdlmsg_set_binding_bindingop_binopinput_soapbody_encodingstyle(WSDLMSG_PTR msg,char *encodingStyle);
int wsdlmsg_set_binding_bindingop_binopinput_soapbody_nameSpace(WSDLMSG_PTR msg,char *nameSpace);
int wsdlmsg_set_binding_bindingop_binopinput_soapbody_use(WSDLMSG_PTR msg,char *use);
int wsdlmsg_set_binding_bindingop_binopoutput_soapbody_encodingstyle(WSDLMSG_PTR msg,char *encodingStyle);
int wsdlmsg_set_binding_bindingop_binopoutput_soapbody_nameSpace(WSDLMSG_PTR msg,char *nameSpace);
int wsdlmsg_set_binding_bindingop_binopoutput_soapbody_use(WSDLMSG_PTR msg,char *use);
int wsdlmsg_set_binding_bindingop_soapop_soapaction(WSDLMSG_PTR msg,char *action);
int wsdlmsg_set_binding_bindingop_soapop_style(WSDLMSG_PTR msg,char *style);
int wsdlmsg_set_service_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_service_port(WSDLMSG_PTR msg,PORT *port);
int wsdlmsg_set_service_port_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_service_port_binding(WSDLMSG_PTR msg,char *binding);
int wsdlmsg_set_service_port_address(WSDLMSG_PTR msg,ADDRESS *address);
int wsdlmsg_set_service_port_address_location(WSDLMSG_PTR msg,char *location);
int wsdlmsg_set_message_part(WSDLMSG_PTR msg,PART *parts);
int wsdlmsg_set_message_part_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_message_part_type(WSDLMSG_PTR msg,char *type);
int wsdlmsg_set_message_part_element(WSDLMSG_PTR msg,char *element);
int wsdlmsg_set_porttype_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_porttype_operation(WSDLMSG_PTR msg,OPERATION *operations);
int wsdlmsg_set_porttype_operation_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_porttype_operation_inputs(WSDLMSG_PTR msg,OPERATION_ELEMENT *inputs);
int wsdlmsg_set_porttype_operation_outputs(WSDLMSG_PTR msg,OPERATION_ELEMENT *outputs);
int wsdlmsg_set_porttype_operation_faults(WSDLMSG_PTR msg,OPERATION_ELEMENT *faults);
int wsdlmsg_set_porttype_operation_inputs_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_porttype_operation_inputs_message(WSDLMSG_PTR msg,char *message);
int wsdlmsg_set_porttype_operation_outputs_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_porttype_operation_outputs_message(WSDLMSG_PTR msg,char *message);
int wsdlmsg_set_porttype_operation_faults_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_set_porttype_operation_faults_message(WSDLMSG_PTR msg,char *message);

MESSAGE_STR *wsdlmsg_get_message(WSDLMSG_PTR msg);
PORT_TYPE *wsdlmsg_get_port_type(WSDLMSG_PTR msg);
SERVICE *wsdlmsg_get_service(WSDLMSG_PTR msg);
BINDING *wsdlmsg_get_binding(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_name(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_type(WSDLMSG_PTR msg);
SOAPBINDING *wsdlmsg_get_binding_soapbinding(WSDLMSG_PTR msg);
BINDING_OP *wsdlmsg_get_binding_bindingop(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_bindingop_name(WSDLMSG_PTR msg);
SOAP_OP *wsdlmsg_get_binding_bindingop_soapop(WSDLMSG_PTR msg);
BIN_OP_INPUT *wsdlmsg_get_binding_bindingop_binopinput(WSDLMSG_PTR msg);
BIN_OP_OUTPUT *wsdlmsg_get_binding_bindingop_binopoutput(WSDLMSG_PTR msg);
SOAP_BODY *wsdlmsg_get_binding_bindingop_binopinput_soapbody(WSDLMSG_PTR msg);
SOAP_BODY *wsdlmsg_get_binding_bindingop_binopoutput_soapbody(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_bindingop_binopinput_soapbody_encodingstyle(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_bindingop_binopinput_soapbody_nameSpace(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_bindingop_binopinput_soapbody_use(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_bindingop_binopoutput_soapbody_encodingstyle(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_bindingop_binopoutput_soapbody_nameSpace(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_bindingop_binopoutput_soapbody_use(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_bindingop_soapop_soapaction(WSDLMSG_PTR msg);
char *wsdlmsg_get_binding_bindingop_soapop_style(WSDLMSG_PTR msg);
char *wsdlmsg_get_service_name(WSDLMSG_PTR msg);
PORT *wsdlmsg_get_service_port(WSDLMSG_PTR msg);
int wsdlmsg_get_service_port_name(WSDLMSG_PTR msg,char *name);
int wsdlmsg_get_service_port_binding(WSDLMSG_PTR msg,char *binding);
int wsdlmsg_get_service_port_address(WSDLMSG_PTR msg,ADDRESS *address);
char *wsdlmsg_get_service_port_address_location(WSDLMSG_PTR msg);
PART *wsdlmsg_get_message_part(WSDLMSG_PTR msg);
char *wsdlmsg_get_message_part_name(WSDLMSG_PTR msg);
char *wsdlmsg_get_message_part_type(WSDLMSG_PTR msg);
char *wsdlmsg_get_message_part_element(WSDLMSG_PTR msg);
char *wsdlmsg_get_porttype_name(WSDLMSG_PTR msg);
OPERATION *wsdlmsg_get_porttype_operation(WSDLMSG_PTR msg);
OPERATION_ELEMENT *wsdlmsg_get_porttype_operation_inputs(WSDLMSG_PTR msg);
OPERATION_ELEMENT *wsdlmsg_get_porttype_operation_outputs(WSDLMSG_PTR msg);
OPERATION_ELEMENT *wsdlmsg_get_porttype_operation_faults(WSDLMSG_PTR msg);
char *wsdlmsg_get_porttype_operation_inputs_name(WSDLMSG_PTR msg);
char *wsdlmsg_get_porttype_operation_inputs_message(WSDLMSG_PTR msg);
char *wsdlmsg_get_porttype_operation_outputs_name(WSDLMSG_PTR msg);
char *wsdlmsg_get_porttype_operation_outputs_message(WSDLMSG_PTR msg);
char *wsdlmsg_get_porttype_operation_faults_name(WSDLMSG_PTR msg);
char *wsdlmsg_get_porttype_operation_faults_message(WSDLMSG_PTR msg);
//hasjdfhjkashdjk
#endif
