#ifndef _SOCKET_H_
#define _SOCKET_H_

#include "msg/msg.h"
#include "event/listeners.h"
#include "unit/unit.h"

#define STR_CONTENTLENGTH "Content-Length"

struct listener;
struct unit;
struct parser;

typedef struct socket* (*FUNC_SOCKET_FACTORY)(struct unit*);

typedef struct socket {
	struct unit* unit;
	
	// socket is a msg subscriber
	void (*func_msg_received)(struct msg*, struct listener*);
	void* (*func_send)(void*);
	
	
	// socket is a msg publisher
	void (*func_publish_msg)(struct msg*, struct socket*);
	struct listener_list_entry* msg_listeners;
	

	int local_port;
	
	FUNC_SOCKET_FACTORY factory;
	
	void* (*func_listen)(void*);
	
} SOCKET;
typedef SOCKET* SOCKET_PTR;

typedef struct socket_thread_data {
	MSG_PTR msg;
	struct socket* socket;
} SOCKETTHREADDATA;
typedef SOCKETTHREADDATA* SOCKETTHREADDATA_PTR;


typedef struct socket_listen_thread_data {
	struct socket* socket;
	char* addr;
	unsigned short port;
} SOCKETLISTENTHREADDATA;
typedef SOCKETLISTENTHREADDATA* SOCKETLISTENTHREADDATA_PTR;




// from UDPMCSocket.c
SOCKET_PTR udpmcsocket_create(struct unit* u);
SOCKET_PTR udp_socket_create(struct unit* u);

void socket_listen(SOCKET_PTR socket, char* addr, unsigned short port );
void* udpmcsocket_send(void* ptr);
	
// from socket.c
SOCKET_PTR socket_create(struct unit* u);
SOCKET_PTR socket_create_without_unit();

// socket is a msg subscriber 
void socket_msg_received(MSG_PTR msg, struct listener* node);

// socket is a msg publisher
int socket_add_msglistener(SOCKET_PTR socket, struct listener* evt_list);
int socket_add_msglistener_unit(SOCKET_PTR socket, struct unit* u);
int socket_add_msglistener_parser(SOCKET_PTR socket, struct parser* p);
void socket_publish_msg(MSG_PTR msg, SOCKET_PTR socket);

int socket_get_localport(SOCKET_PTR socket);
int socket_set_localport(SOCKET_PTR socket, int local_port);

void* tcp_socket_listen(void* ptr);
void* tcp_socket_send(void* ptr);
SOCKET_PTR tcp_socket_create(struct unit*  u);
SOCKET_PTR http_socket_create(struct unit* u);

#endif //_SOCKET_H_

