#ifndef _UNIT_H_
#define _UNIT_H_

#include "event/event.h"
#include "event/listeners.h"
#include "msg/msg.h"
#include "socket/socket.h"
#include "parser/parser.h"
#include "composer/composer.h"
#include "event/eventqueue_table.h"

#include "engine/engine.h"
#include "engine/action.h"


struct listener;


typedef struct map_entry_socket {
	char* key;
	struct socket* socket;
	struct map_entry_socket* next;
} MAPENTRYSOCKET;	
typedef MAPENTRYSOCKET* MAPENTRYSOCKET_PTR;	

typedef struct map_entry_parser {
	char* key;
	struct parser* parser;
	struct map_entry_parser* next;
} MAPENTRYPARSER;	
typedef MAPENTRYPARSER* MAPENTRYPARSER_PTR;	


typedef struct map_entry_composer {
	char* key;
	struct composer* composer;
	struct map_entry_composer* next;
} MAPENTRYCOMPOSER;	
typedef MAPENTRYCOMPOSER* MAPENTRYCOMPOSER_PTR;	

typedef struct unit {
	MAPENTRYPARSER_PTR parsers;
	MAPENTRYCOMPOSER_PTR composers;
	MAPENTRYSOCKET_PTR sockets;
	
	struct parser* current_parser;
	struct composer* current_composer;


	// unit is an evt subscriber 
	void (*func_event_received)(struct event*, struct listener* node);
	
	// unit is an evt publisher 
	void (*func_publish_event)(struct event*, struct unit*);
	struct listener_list_entry*  evt_listeners;
	
	// unit is a msg subscriber 
	void (*func_msg_received)(struct msg*, struct listener*);
	
	// unit is a msg publisher
	void (*func_publish_msg)(struct msg*, struct unit* /*socket* */);
	struct listener_list_entry* msg_listeners;
	
	EVENTQUEUE_TABLE_PTR event_table;
	
	ENGINE_PTR engine;
	ACTION_PTR actions;
} UNIT;
typedef UNIT* UNIT_PTR;

// unit is a msg subscriber 
void unit_msg_received(struct msg* msg, struct listener* node);

// unit is a msg publisher
void unit_publish_msg(struct msg* msg, UNIT_PTR unit);
int unit_add_msglistener(UNIT_PTR unit, struct listener* evt_list);
int unit_add_msglistener_unit(UNIT_PTR unit, struct unit* u);
int unit_add_msglistener_parser(UNIT_PTR unit, struct parser* p);


// unit is an evt subscriber 
void unit_event_received(EVENT_PTR evt, struct listener* node);

// unit is an evt publisher 
int unit_add_evtlistener(UNIT_PTR unit, struct listener* evt_list);
int unit_add_evtlistener_parser(UNIT_PTR unit, struct parser* p);
int unit_add_evtlistener_composer(UNIT_PTR unit, struct composer* c);
int unit_add_evtlistener_unit(UNIT_PTR unit, UNIT_PTR u);
int unit_add_evtlistener_unit_proxy(UNIT_PTR unit, UNIT_PTR u);
void unit_publish_event(EVENT_PTR evt, UNIT_PTR unit);



int unit_addparser(UNIT_PTR unit, char* key, struct parser* p);
int unit_addcomposer(UNIT_PTR unit, char* key, struct composer* c);
int unit_addsocket(UNIT_PTR unit, char* key, struct socket* s);
struct socket* unit_getsocket(UNIT_PTR unit, char* key);
struct composer* unit_getcomposer(UNIT_PTR unit, char* key);
struct parser* unit_getparser(UNIT_PTR unit, char* key);

int unit_addaction_function1(UNIT_PTR unit, char* name, prototype_function1 function);
int unit_addaction_function2(UNIT_PTR unit, char* name, prototype_function2 function);


UNIT_PTR unit_create(char* sm_filename, ACTION_PTR extra_actions);
UNIT_PTR slpunit_create(char* sm_filename);
UNIT_PTR upnpunit_create(char* sm_filename);
UNIT_PTR wsdunit_create(char* sm_filename);
UNIT_PTR soapunit_create(char* sm_filename);
UNIT_PTR rmiunit_create(char* sm_filename);

UNIT_PTR upnpunit_create2(char* sm_filename);
UNIT_PTR slpunit_create2(char* sm_filename);
UNIT_PTR upnpproxy_unit_create2(char* sm_filename);
UNIT_PTR rmiproxy_unit_create2(char* sm_filename);
UNIT_PTR soapunit_create2(char* sm_filename);
UNIT_PTR rmiunit_create2(char* sm_filename);
UNIT_PTR wsdunit_create2(char* sm_filename);
UNIT_PTR wsdproxy_unit_create2(char* sm_filename);

void unit_dispatch_evt_to_listeners(EVENT_PTR evt, void* u);
void unit_dispatch_evt_to_composers(EVENT_PTR evt, void* u);
void unit_dispatch_evt_to_units(EVENT_PTR evt, void* u);
void unit_dispatch_evt_to_unit_proxy(EVENT_PTR evt, void* u);

#endif //_UNIT_H_
