#ifndef _ATTRIBUTES_H_
#define _ATTRIBUTES_H_

#include <vm/jclass.h>
#include <kazlib/hash.h>

#define SOURCE_FILE 	"SourceFile"
#define CODE			"Code"
#define EXCEPTIONS		"Exceptions"
#define SYNTHETIC		"Synthetic"
#define CONSTANT_VALUE	"ConstantValue"
#define INNER_CLASSE   	"InnerClasses"

#define LINE_NUMBER_TABLE   "LineNumberTable"
#define LOCAL_VARIABLE_TABLE   "LocalVariableTable"


enum AttributeTag {
	CODE_ID=0,
	EXCEP_ID=1, 
	SYNTH_ID=2,
	CONST_ID=3,
	LINENUM_ID=4,
	LOCALVAR_ID=5
};

typedef enum AttributeTag ATAG;

struct Instr {
	uint16 	pc;		
	uint8 	opcode;
	uint8 	length;
	uint8* 	index;	
	uint8   arg;	
	char* 	mnem;
};

struct Exception {
	uint16 start_pc;
	uint16 end_pc;
	uint16 handler;
	uint16* index; 
};

typedef struct Exception EXCEPTION;
typedef EXCEPTION* EXCEPTION_PTR;

struct Label {
	char* label;
	uint16 pc; 
};

typedef struct Label LABEL;
typedef LABEL* LABEL_PTR;

typedef struct Instr INSTR;
typedef INSTR* INSTR_PTR;

struct AttrCode{
	uint16	current_pc;
	uint16  maxStack;	
	uint16	maxLocal;
	list_t* listCode;
	list_t* listException;
	hash_t* hashLabel;
	list_t* listAttributes;
};

typedef struct AttrCode ATTR_CODE;
typedef ATTR_CODE*	ATTR_CODE_PTR;

struct AttrException{
	uint16 nb; 		
	uint16** tab; 	
};

typedef struct AttrException ATTR_EXCEPTION;
typedef ATTR_EXCEPTION*	ATTR_EXCEPTION_PTR;

struct AttrConstant{
	uint16* constant_value_index; 
};

typedef struct AttrConstant ATTR_CONSTANT;
typedef ATTR_CONSTANT* ATTR_CONSTANT_PTR;

// LOCAL VARIABLE TABLE IS AN ATTRIBUTE INCLUDED IN ATTRIBUTE CODE
struct LocalVarEntry {
	POOL_PTR pool;
	uint16	start_pc;
	uint16* name_index;
	uint16* descriptor_index;
	uint16	index;
};
typedef struct LocalVarEntry LOCAL_VAR_ENTRY;
typedef LOCAL_VAR_ENTRY*	LOCAL_VAR_ENTRY_PTR;


struct AttrLocalVarTable{
	uint16	table_length;
	list_t* local_var_table;
};

typedef struct AttrLocalVarTable ATTR_LOCAL_VAR_TABLE;
typedef ATTR_LOCAL_VAR_TABLE*	ATTR_LOCAL_VAR_TABLE_PTR;


// LINE NUMBER TABLE IS AN ATTRIBUTE INCLUDED IN ATTRIBUTE CODE
struct LineNumberEntry {
	uint16	start_pc;
	uint16	line_number;
};
typedef struct LineNumberEntry LINE_NUMBER_ENTRY;
typedef LINE_NUMBER_ENTRY*	LINE_NUMBER_ENTRY_PTR;


struct AttrLineNumberTable{
	uint16	table_length;
	list_t* line_number_table;
};

typedef struct AttrLineNumberTable ATTR_LINE_NUMBER_TABLE;
typedef ATTR_LINE_NUMBER_TABLE*	ATTR_LINE_NUMBER_TABLE_PTR;



struct Attribute{
	POOL_PTR pool;
	ATAG	tag;
	uint16* name_index;
	uint32  attr_length;
	union {
		ATTR_CODE_PTR code;
		ATTR_EXCEPTION_PTR exception;
		ATTR_CONSTANT_PTR constant;
		ATTR_LINE_NUMBER_TABLE_PTR line_num_table;
		ATTR_LOCAL_VAR_TABLE_PTR local_var_table;
	} info;
};

typedef struct Attribute ATTR;
typedef ATTR*	ATTR_PTR;


extern void code_add_ref_instr(ATTR_PTR attr,uint8 op,TAG t,const char* className,const char* varName, const char* type);
extern void code_add_field_instr(ATTR_PTR attrCode,uint8 opcode,const char* className,const char* varName, const char* type);
extern void code_add_method_instr(ATTR_PTR attrCode,uint8 opcode,const char* returnType,const char*methodName,const char* signature);
extern void code_add_single_instr(ATTR_PTR attrCode,uint8 opcode);
extern void code_add_type_instr(ATTR_PTR attr,uint8 op,const char* type);
extern void code_add_ldc(ATTR_PTR attr,uint8 type,void* data);

extern void attr_code_set_max(ATTR_PTR attrCode,uint16 maxLocal,uint16 maxStack);

extern	INSTR_PTR code_add_jump_instr(ATTR_PTR attrCode,uint8 opcode,char* label);
INSTR_PTR code_new_x1_instr(ATTR_PTR attr,uint8 op,uint8* index);




ATTR_PTR parse_attr_charPtr(char** stream, POOL_PTR pool);

#endif
