#include "composer/composer.h"
#include "composer/ssdpcomposer.h"
#include "unit/unit.h"
#include "protocol/ssdpmsg.h"

#include "io/io.h"
#include <stdio.h>
#include "event/eventqueue_table.h"
#include <pthread.h>

#include "monitor/tools.h"


MSG_PTR ssdpcomposer_ssdp_request(SSDPMSG_PTR ssdpmsg)
{

	MSG_PTR request_msg = msg_create("", 0, get_host_ip(), 1000, ssdpmsg_get_dest_addr(ssdpmsg), ssdpmsg_get_dest_port(ssdpmsg), NULL);	

	
	request_msg->length = 0;
	
	// calculate length first line
	char* method = ssdpmsg_get_method(ssdpmsg);
	char* version = ssdpmsg_get_version(ssdpmsg);
	char* uri =	ssdpmsg_get_uri(ssdpmsg);
	request_msg->length += strlen(method) + strlen(version) + 3 /* 3 chars for " * " */ + strlen("\r\n") /* for \r\n */;  
	
	//	calculate length headers
	SSDP_HEADER_PTR headers = NULL;
	for (headers = ssdpmsg_get_headers(ssdpmsg); headers != NULL; headers = headers->next)
		request_msg->length += strlen(headers->name) + strlen(headers->value) + 2 /* ": " */ +strlen("\r\n") /* \r\n */;
		
	// for final \0 
	//request_msg->length += 1; 
	request_msg->length += strlen("\r\n");
		
	request_msg->data = malloc(request_msg->length * sizeof(char));
	
	// first line
	/*strncpy(request_msg->data, method, strlen(method));
	strncat(request_msg->data, " * ", 3);
	strncat(request_msg->data, version, strlen(version));
	strncat(request_msg->data, "\n", 1);
	*/
	char* index_msg_data = request_msg->data;
	memcpy(index_msg_data, method, strlen(method));
	index_msg_data += strlen(method)*sizeof(char);
	
	memcpy(index_msg_data, " * ", strlen(" * "));
	index_msg_data += strlen(" * ")*sizeof(char);
	
	memcpy(index_msg_data, version, strlen(version));
	index_msg_data += strlen(version)*sizeof(char);
	
	memcpy(index_msg_data, "\r\n", strlen("\r\n"));
	index_msg_data += strlen("\r\n")*sizeof(char);
	
	// add headers
	for (headers = ssdpmsg_get_headers(ssdpmsg); headers != NULL; headers = headers->next)
	{
		/*
		strncat(request_msg->data, headers->name, strlen(headers->name));
		strncat(request_msg->data, ": ", 2);
		strncat(request_msg->data, headers->value, strlen(headers->value));
		strncat(request_msg->data, "\n", 1);
		*/
		memcpy(index_msg_data, headers->name, strlen(headers->name));
		index_msg_data += strlen(headers->name)*sizeof(char);
		
		memcpy(index_msg_data, ": ", strlen(": "));
		index_msg_data += strlen(": ")*sizeof(char);

		memcpy(index_msg_data, headers->value, strlen(headers->value));
		index_msg_data += strlen(headers->value)*sizeof(char);
		
		memcpy(index_msg_data, "\r\n", strlen("\r\n"));
		index_msg_data += strlen("\r\n")*sizeof(char);
	}
	//strncat(request_msg->data, "\0", 1);
	memcpy(index_msg_data, "\r\n", strlen("\r\n"));
	index_msg_data += strlen("\r\n")*sizeof(char);
	
	return request_msg;
}

MSG_PTR _ssdpcomposer_ssdp_request(SSDPMSG_PTR ssdpmsg)
{

	MSG_PTR request_msg = msg_create("", 0, get_host_ip()/*"192.168.2.2"*/, 1000, ssdpmsg_get_dest_addr(ssdpmsg), ssdpmsg_get_dest_port(ssdpmsg), NULL);	

	
	request_msg->length = 0;
	
	// calculate length first line
	char* method = ssdpmsg_get_method(ssdpmsg);
	char* version = ssdpmsg_get_version(ssdpmsg);
	char* uri =	ssdpmsg_get_uri(ssdpmsg);
	request_msg->length += strlen(method) + strlen(version) + 3 /* 3 chars for " * " */ + 1 /* for \n */;  
	
	//	calculate length headers
	SSDP_HEADER_PTR headers = NULL;
	for (headers = ssdpmsg_get_headers(ssdpmsg); headers != NULL; headers = headers->next)
		request_msg->length += strlen(headers->name) + strlen(headers->value) + 2 /* ": " */ +1 /* \n */;
		
	// for final \0 
	request_msg->length += 1; 
		
	request_msg->data = malloc(request_msg->length);
	
	// first line
	strncpy(request_msg->data, method, strlen(method));
	strncat(request_msg->data, " * ", 3);
	strncat(request_msg->data, version, strlen(version));
	strncat(request_msg->data, "\n", 1);
	
	// add headers
	for (headers = ssdpmsg_get_headers(ssdpmsg); headers != NULL; headers = headers->next)
	{
		strncat(request_msg->data, headers->name, strlen(headers->name));
		strncat(request_msg->data, ": ", 2);
		strncat(request_msg->data, headers->value, strlen(headers->value));
		strncat(request_msg->data, "\n", 1);
	}
	strncat(request_msg->data, "\0", 1);
	
	return request_msg;
}


MSG_PTR ssdpcomposer_ssdp_reply(SSDPMSG_PTR ssdpmsg)
{

	MSG_PTR reply_msg = msg_create("", 0, get_host_ip()/*"192.168.2.2"*/, 1000, ssdpmsg_get_dest_addr(ssdpmsg), ssdpmsg_get_dest_port(ssdpmsg), NULL);	

	
	reply_msg->length = 0;
	
	// calculate length first line
	char* version = ssdpmsg_get_version(ssdpmsg);
	char* code = ssdpmsg_get_code(ssdpmsg);
	reply_msg->length += strlen(version) + 1 + strlen(code) + 1 + strlen("OK") + 1 /* for \n */;  
	
	//	calculate length headers
	SSDP_HEADER_PTR headers = NULL;
	for (headers = ssdpmsg_get_headers(ssdpmsg); headers != NULL; headers = headers->next)
		reply_msg->length += strlen(headers->name) + strlen(headers->value) + 2 /* ": " */ +1 /* \n */;
	
	// for final \0 
	reply_msg->length += 1; 
	
	reply_msg->data = (char*)malloc(reply_msg->length);
	
	// first line
	strncpy(reply_msg->data, version, strlen(version));
	strncat(reply_msg->data, " ", 1);
	strncat(reply_msg->data, code, strlen(code));
	strncat(reply_msg->data, " OK", 3);
	strncat(reply_msg->data, "\n", 1);
	
	// add headers
	for (headers = ssdpmsg_get_headers(ssdpmsg); headers != NULL; headers = headers->next)
	{
		strncat(reply_msg->data, headers->name, strlen(headers->name));
		strncat(reply_msg->data, ": ", 2);
		strncat(reply_msg->data, headers->value, strlen(headers->value));
		strncat(reply_msg->data, "\n", 1);
	}
	strncat(reply_msg->data, "\0", 1);
	
	return reply_msg;
}


void *ssdp_run_composer(void* ptr)
{

	fprintf(stdout,"ssdp_run_composer: starts. \n");
	static int already_sent = 0;
	
	COMPOSETHREADDATA_PTR data = (COMPOSETHREADDATA_PTR)ptr;
	
	EVENTQUEUE_TABLE_ENTRY_PTR entry = (EVENTQUEUE_TABLE_ENTRY_PTR)data->entry_event_queue;
	COMPOSER_PTR composer = (COMPOSER_PTR)data->composer;
	
pthread_mutex_lock(&(entry->mutex));
	
	
	// Add the identifier of the thread composer to the eventqueue_table 
	pthread_t thread_composer = pthread_self();
	//entry->composer_evtlistener_thread = (void*)thread_composer;
//	eventqueue_table_add_composer_thread(entry, thread_composer);
//	// if there are other composers currently active on the entry, wait for the last one to terminate
//	if (eventqueue_table_get_composer_thread_list_length(entry) > 1)
//		pthread_join(eventqueue_table_get_last_composer_thread(entry), NULL);
	

	//pthread_t thread_event_src = pthread_self();
	//fprintf(stdout, "\nSSDP PTHREAD composer = %p\n", thread_event_src);
	
	//ssdp_parse_msg(data->parser, data->msg->data, data->msg->length, thread_event_src);
	
	//eventqueue_table_print(eventqueue_table);
	//fprintf(stdout,"table contains 13,456 = %d\n", eventqueue_table_contain_entry(eventqueue_table, 13,456));
	
	
	SSDP_SERVICE_DISCOVER_TABLE services_table;
	ssdp_service_discover_table_init(&services_table);
	
	//char* reply_dest_addr;
	//int reply_dest_port;
	//SSDPMSG_PTR ssdpmsg = ssdpmsg_create();
	
	fprintf(stdout,"ssdp_run_composer: starts. number of events=%d\n",entry->event_queue->size);
	
	
	while (1)
	{
//		fprintf(stdout,"ssdp_run_composer: starts. number of events=%d\n",entry->event_queue->size);
		
		EVENT_PTR evt = eventqueue_table_get_event(entry);
		if (evt != NULL)
		{
			fprintf(stdout,"ssdp_run_composer: evt is type=%d, src=%p\n",  evt->type,evt->stream_id);
			event_print(evt);  
			
			switch (evt->type) 
			{
			case SDP_SOURCE_ADDR:
			{
				fprintf(stdout,"ssdp_run_composer: before table.n_services=%d, \n",services_table.n_services);
				SSDPMSG_PTR ssdpmsg;
				if (ssdp_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					ssdpmsg = ssdp_service_discover_table_get_ssdpmsg_by_stream_id(services_table, evt->stream_id);
				else
				{
					ssdpmsg = ssdpmsg_create();
					ssdp_service_discover_table_add_entry(&services_table, evt->stream_id, ssdpmsg);
				}
				ssdpmsg_set_dest_addr(ssdpmsg, evt->value.value_string);
				
				fprintf(stdout,"ssdp_run_composer: after table.n_services=%d, \n",services_table.n_services);
			}	
				//reply_dest_addr = strdup(evt->value.value_string);
				//ssdpmsg_set_dest_addr(ssdpmsg, reply_dest_addr);
				break;		
			case SDP_SOURCE_PORT:
			{
				SSDPMSG_PTR ssdpmsg;
				if (ssdp_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					ssdpmsg = ssdp_service_discover_table_get_ssdpmsg_by_stream_id(services_table, evt->stream_id);
				else
				{
					ssdpmsg = ssdpmsg_create();
					ssdp_service_discover_table_add_entry(&services_table, evt->stream_id, ssdpmsg);
				}
				ssdpmsg_set_dest_port(ssdpmsg, evt->value.value_int);
			}
				//reply_dest_port = evt->value.value_int;
				//ssdpmsg_set_dest_port(ssdpmsg, reply_dest_port);
				break;	
			
			case SDP_REQ_LANGAGE:
	    		//	ssdpmsg_set_lang_tag(ssdpmsg, evt->value.value_string);
				break;
			case SDP_REQ_SCOPE:
				//ssdpmsg_set_scope_list(ssdpmsg, evt->value.value_string);
				break;	
			case SDP_REQ_PREDICAT:
				//ssdpmsg_set_predicate(ssdpmsg, evt->value.value_string);
				break;	
			case SDP_REQ_REQUESTER:
				//ssdpmsg_set_pr_list(ssdpmsg, evt->value.value_string);
				break;	

			case SDP_REQ_SERVICE_TYPE:
			{
				/*
				if (strstr(evt->value.value_string, "service:"))
				{
					int len_serv_type = strlen(evt->value.value_string) - strlen("service:") + strlen("device:");
					char* st = malloc((strlen("urn:schemas-upnp-org:") + len_serv_type + strlen(":1") + 1) * sizeof(char));
					strcpy(st, "urn:schemas-upnp-org:");
					strcat(st, "device:");
					strcat(st, (evt->value.value_string)+(strlen("service:")*sizeof(char)));
					strcat(st, ":1");
					strcat(st, "\0" );
					ssdpmsg_add_header(ssdpmsg, "ST", st);
				}
					*/	
				SSDPMSG_PTR ssdpmsg;
				if (ssdp_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					ssdpmsg = ssdp_service_discover_table_get_ssdpmsg_by_stream_id(services_table, evt->stream_id);
				else
				{
					ssdpmsg = ssdpmsg_create();
					ssdp_service_discover_table_add_entry(&services_table, evt->stream_id, ssdpmsg);
				}
				
				ssdp_service_discover_table_set_service_type(&services_table, evt->stream_id, evt->value.value_string);	
				fprintf(stdout,"ssdp_run_composer: set_service_type=%s, \n",evt->value.value_string);
			}
				break;	
			case SDP_REQ_ID:
				//ssdpmsg_set_xid(ssdpmsg, evt->value.value_int);
				break;		
			case SDP_VERSION:
				//ssdpmsg_set_version(ssdpmsg, evt->value.value_int);
				break;		
			case SDP_SERVICEPROXY_URL_CTRL:
			{
				SSDPMSG_PTR ssdpmsg;
				fprintf(stdout,"ssdp_run_composer: before table.n_services=%d, \n",services_table.n_services);
				
				if (ssdp_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					ssdpmsg = ssdp_service_discover_table_get_ssdpmsg_by_stream_id(services_table, evt->stream_id);
				else
				{
					ssdpmsg = ssdpmsg_create();
					ssdp_service_discover_table_add_entry(&services_table, evt->stream_id, ssdpmsg);
				}
				
				ssdp_service_discover_table_set_location(&services_table, evt->stream_id, evt->value.value_string);
				fprintf(stdout,"ssdp_run_composer: after table.n_services=%d, \n",services_table.n_services);
			}
			//	ssdpmsg_add_header(ssdpmsg, "LOCATION",  evt->value.value_string);
				break;
			case SDP_SEND_SERVICE_REPLY:
				if (ssdp_service_discover_table_contains_stream_id(services_table, evt->stream_id))
				{
					SSDPMSG_PTR ssdpmsg = ssdp_service_discover_table_get_ssdpmsg_by_stream_id(services_table, evt->stream_id);
					
					fprintf(stdout, "SDP_SEND_SERVICE_REPLY\n");
					
					ssdpmsg_set_isReply(ssdpmsg);
					ssdpmsg_set_version(ssdpmsg, "HTTP/1.1");
					ssdpmsg_set_code(ssdpmsg, "200");
	
					ssdpmsg_add_header(ssdpmsg, "ST", "upnp:rootdevice");
					
					char st[255];
					sprintf(st, "uuid:AmigoDevice::urn:schemas-upnp-org:device:%s:1", ssdp_service_discover_table_get_service_type(services_table, evt->stream_id));
					ssdpmsg_add_header(ssdpmsg, "USN", st);
					//ssdpmsg_set_dest_addr(ssdpmsg, "239.255.255.250");
					//ssdpmsg_set_dest_port(ssdpmsg, 1900);
					
					char* location = ssdp_service_discover_table_get_location(services_table, evt->stream_id);
					if (location != NULL)
						ssdpmsg_add_header(ssdpmsg, "LOCATION",  location);
					else
					{
						char* location_nopath = ssdp_service_discover_table_get_location_url_nopath(services_table, evt->stream_id);
						char* location_path = ssdp_service_discover_table_get_location_url_path(services_table, evt->stream_id);
						
						if (location_nopath != NULL)
						{
							location = strdup(location_nopath);
							if (location_path != NULL)
								strcat(location ,location_path);
							ssdpmsg_add_header(ssdpmsg, "LOCATION",  location);
						}
					}
					
					MSG_PTR msg = ssdpcomposer_ssdp_reply(ssdpmsg);
					
					composer_publish_msg(msg, composer);
				}	
				break;
				
				
			//case SDP_STOP:
			case SDP_SEND_SERVICE_REQUEST:
if (!already_sent)
				{
	//already_sent =1;	
			
				
				if (ssdp_service_discover_table_contains_stream_id(services_table, evt->stream_id))
				{
					SSDPMSG_PTR ssdpmsg = ssdp_service_discover_table_get_ssdpmsg_by_stream_id(services_table, evt->stream_id);
					
					ssdpmsg_set_isSearch(ssdpmsg);
					ssdpmsg_set_method(ssdpmsg, "M-SEARCH");
					ssdpmsg_set_version(ssdpmsg, "HTTP/1.1");
					ssdpmsg_set_uri(ssdpmsg, "*");
					ssdpmsg_add_header(ssdpmsg, "HOST", "239.255.255.250:1900");
					ssdpmsg_add_header(ssdpmsg, "MX", "3");
					ssdpmsg_add_header(ssdpmsg, "MAN", "\"ssdp:discover\"");

					ssdpmsg_set_dest_addr(ssdpmsg, "239.255.255.250");
					ssdpmsg_set_dest_port(ssdpmsg, 1900);
					
					char* service_type = ssdp_service_discover_table_get_service_type(services_table, evt->stream_id);
					if (service_type != NULL)
					{
						char st[255];
						sprintf(st, "urn:schemas-upnp-org:device:%s:1",service_type);
						ssdpmsg_add_header(ssdpmsg, "ST", st);
					}
					
					MSG_PTR msg = ssdpcomposer_ssdp_request(ssdpmsg);
					
					fprintf(stdout, "ssdp_run_composer: (before remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
					COMPOSERTHREAD_LIST_ENTRY_PTR first_composer_thread  = eventqueue_table_remove_first_composer_thread(entry);
					// set current_socket_reply_port for ssdp parser to find the right entry on the table
					entry->current_socket_reply_port = first_composer_thread->socket_reply_port;	

			
					composer_publish_msg(msg, composer);
					pthread_cond_signal(&(first_composer_thread->cond));
					fprintf(stdout, "ssdp_run_composer: (after remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
					pthread_mutex_unlock(&(entry->mutex));
					
					fprintf(stdout, "ssdp_run_composer: signal cond var %p\n", &(first_composer_thread->cond));
					return;

				}
				

				
				
			/*
			if (!already_sent)
			{
				already_sent =1;
				ssdpmsg_set_isSearch(ssdpmsg);
				ssdpmsg_set_method(ssdpmsg, "M-SEARCH");
				ssdpmsg_set_version(ssdpmsg, "HTTP/1.1");
				ssdpmsg_set_uri(ssdpmsg, "*");
				ssdpmsg_add_header(ssdpmsg, "HOST", "239.255.255.250:1900");
				ssdpmsg_add_header(ssdpmsg, "MX", "3");
				ssdpmsg_add_header(ssdpmsg, "MAN", "\"ssdp:discover\"");

				ssdpmsg_set_dest_addr(ssdpmsg, "239.255.255.250");
				ssdpmsg_set_dest_port(ssdpmsg, 1900);
				
				
				MSG_PTR msg = ssdpcomposer_ssdp_request(ssdpmsg);
				
				fprintf(stdout, "ssdp_run_composer: (before remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
				COMPOSERTHREAD_LIST_ENTRY_PTR first_composer_thread  = eventqueue_table_remove_first_composer_thread(entry);
				// set current_socket_reply_port for ssdp parser to find the right entry on the table
				entry->current_socket_reply_port = first_composer_thread->socket_reply_port;	

		
				composer_publish_msg(msg, composer);
				pthread_cond_signal(&(first_composer_thread->cond));
				fprintf(stdout, "ssdp_run_composer: (after remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
				pthread_mutex_unlock(&(entry->mutex));
				
				fprintf(stdout, "ssdp_run_composer: signal cond var %p\n", &(first_composer_thread->cond));
				return;
			}
			*/
			}
				break;

			}	
		
		}
		usleep(10);
		
		
		
		
		//else
		//	fprintf(stdout,"ssdp_run_composer: NO evt !\n"); 
	}
}

void ssdp_composer_event_received(EVENT_PTR evt, LISTENER_PTR node)
{
	fprintf(stdout,"SSDP");
	composer_event_received(evt, node);
}


COMPOSER_PTR ssdp_composer_create(UNIT_PTR u)
{
	fprintf(stdout, "Creating SSDP composer\n");
		
	
	COMPOSER_PTR c = composer_create(u);

	c->run_composer = ssdp_run_composer;
	c->factory = ssdp_composer_create;

	c->func_event_received = ssdp_composer_event_received;	
	return c;	
}


void ssdp_service_discover_table_init(SSDP_SERVICE_DISCOVER_TABLE* table)
{
	table->n_services = 0;
	int i =0;
	for (i=0 ; i<100; i++)
	{
		table->services[i].stream_id = -1;
		table->services[i].ssdpmsg = NULL;
		strcpy(table->services[i].location_url_nopath, ""); // = NULL;
		strcpy(table->services[i].location_url_path, ""); // = NULL;
		strcpy(table->services[i].location, ""); // = NULL;
	}	
}

void ssdp_service_discover_table_add_entry(SSDP_SERVICE_DISCOVER_TABLE* table, int stream_id, SSDPMSG_PTR ssdpmsg)
{
	fprintf(stdout, "upnp_service_discover_table_add_entry begin (services_table=%p). (nservices=%d)\n",table, table->n_services);
	
	table->services[table->n_services].stream_id = stream_id;
	table->services[table->n_services].ssdpmsg = ssdpmsg;
	table->n_services += 1;
	
	fprintf(stdout, "upnp_service_discover_table_add_entry end. (nservices=%d) \n",table->n_services );
}

int ssdp_service_discover_table_contains_stream_id(SSDP_SERVICE_DISCOVER_TABLE table, int stream_id)
{
	int i =0;
	for (i=0 ; i<table.n_services; i++)
	{
		fprintf(stdout, "ssdp_service_discover_table_contains_stream_id. str1=%p str2=%p\n",table.services[i].stream_id, stream_id);
		if (table.services[i].stream_id == stream_id)
			return 1;
	}
	return 0;
}


SSDPMSG_PTR ssdp_service_discover_table_get_ssdpmsg_by_stream_id(SSDP_SERVICE_DISCOVER_TABLE table, int stream_id)
{
	int i =0;
	for (i=0 ; i<table.n_services; i++)
	{
		
		fprintf(stdout, "ssdp_service_discover_table_get_ssdpmsg_by_stream_id. str1=%p str2=%p\n",table.services[i].stream_id, stream_id);
		if (table.services[i].stream_id == stream_id)
			return table.services[i].ssdpmsg;
	}
	return NULL;
}

char* ssdp_service_discover_table_get_location_url_path(SSDP_SERVICE_DISCOVER_TABLE table, int stream_id)
{
	int i =0;
	for (i=0 ; i<table.n_services; i++)
	{
		
		fprintf(stdout, "upnp_service_discover_table_get_location_url_path. (nservices=%d) str1=%p str2=%p\n table.services[i].location_url_path=%s",table.n_services, table.services[i].stream_id, stream_id, table.services[i].location_url_path);
		if (table.services[i].stream_id == stream_id)
		{
			//fprintf(stdout, "upnp_service_discover_table_get_location_url_path.FOUND!!!\n");
			return  table.services[i].location_url_path;
		}
		
	}
	return NULL;
}

char* ssdp_service_discover_table_get_location_url_nopath(SSDP_SERVICE_DISCOVER_TABLE table, int stream_id)
{
	int i =0;
	for (i=0 ; i<table.n_services; i++)
	{
		
		fprintf(stdout, "upnp_service_discover_table_get_location_url_path. (nservices=%d) str1=%p str2=%p\n table.services[i].location_url_path=%s",table.n_services, table.services[i].stream_id, stream_id, table.services[i].location_url_path);
		if (table.services[i].stream_id == stream_id)
		{
			//fprintf(stdout, "upnp_service_discover_table_get_location_url_path.FOUND!!!\n");
			return  table.services[i].location_url_nopath;
		}
		
	}
	return NULL;
}

char* ssdp_service_discover_table_get_location(SSDP_SERVICE_DISCOVER_TABLE table, int stream_id)
{
	int i =0;
	for (i=0 ; i<table.n_services; i++)
	{
		
		fprintf(stdout, "upnp_service_discover_table_get_location_url_path. (nservices=%d) str1=%p str2=%p\n table.services[i].location_url_path=%s",table.n_services, table.services[i].stream_id, stream_id, table.services[i].location_url_path);
		if (table.services[i].stream_id == stream_id)
		{
			//fprintf(stdout, "upnp_service_discover_table_get_location_url_path.FOUND!!!\n");
			return  table.services[i].location;
		}
		
	}
	return NULL;
}

char* ssdp_service_discover_table_get_service_type(SSDP_SERVICE_DISCOVER_TABLE table, int stream_id)
{
	int i =0;
	for (i=0 ; i<table.n_services; i++)
	{
		
		fprintf(stdout, "upnp_service_discover_table_get_location_url_path. (nservices=%d) str1=%p str2=%p\n table.services[i].location_url_path=%s",table.n_services, table.services[i].stream_id, stream_id, table.services[i].service_type);
		if (table.services[i].stream_id == stream_id)
		{
			//fprintf(stdout, "upnp_service_discover_table_get_location_url_path.FOUND!!!\n");
			return  table.services[i].service_type;
		}
		
	}
	return NULL;
}

void ssdp_service_discover_table_set_location_url_nopath(SSDP_SERVICE_DISCOVER_TABLE* table, int stream_id, char* location_url_nopath)
{
	int i =0;
	for (i=0 ; i<table->n_services; i++)
	{
		fprintf(stdout, "upnp_service_discover_table_set_location_url_path. (nservices=%d) str1=%p str2=%p location_url_path=%s\n",table->n_services, table->services[i].stream_id, stream_id, location_url_nopath);
		if (table->services[i].stream_id == stream_id)
		{
			strcpy(table->services[i].location_url_nopath ,location_url_nopath);
			fprintf(stdout, "table.services[i].location_url_path set to : %s location_url_path=%s\n",table->services[i].location_url_nopath, location_url_nopath);
		}
	}
}

void ssdp_service_discover_table_set_location_url_path(SSDP_SERVICE_DISCOVER_TABLE* table, int stream_id, char* location_url_path)
{
	int i =0;
	for (i=0 ; i<table->n_services; i++)
	{
		fprintf(stdout, "upnp_service_discover_table_set_location_url_path. (nservices=%d) str1=%p str2=%p location_url_path=%s\n",table->n_services, table->services[i].stream_id, stream_id, location_url_path);
		if (table->services[i].stream_id == stream_id)
		{
			strcpy(table->services[i].location_url_path ,location_url_path);
			fprintf(stdout, "table.services[i].location_url_path set to : %s location_url_path=%s\n",table->services[i].location_url_path, location_url_path);
		}
	}
}

void ssdp_service_discover_table_set_location(SSDP_SERVICE_DISCOVER_TABLE* table, int stream_id, char* location)
{
	int i =0;
	for (i=0 ; i<table->n_services; i++)
	{
		fprintf(stdout, "upnp_service_discover_table_set_location_url_path. (nservices=%d) str1=%p str2=%p location_url_path=%s\n",table->n_services, table->services[i].stream_id, stream_id, location);
		if (table->services[i].stream_id == stream_id)
		{
			strcpy(table->services[i].location ,location);
			fprintf(stdout, "table.services[i].location_url_path set to : %s location_url_path=%s\n",table->services[i].location, location);
		}
	}
}

void ssdp_service_discover_table_set_service_type(SSDP_SERVICE_DISCOVER_TABLE* table, int stream_id, char* service_type)
{
	int i =0;
	for (i=0 ; i<table->n_services; i++)
	{
		fprintf(stdout, "upnp_service_discover_table_set_location_url_path. (nservices=%d) str1=%p str2=%p location_url_path=%s\n",table->n_services, table->services[i].stream_id, stream_id, service_type);
		if (table->services[i].stream_id == stream_id)
		{
			strcpy(table->services[i].service_type ,service_type);
			fprintf(stdout, "table.services[i].location_url_path set to : %s location_url_path=%s\n",table->services[i].service_type, service_type);
		}
	}
}
