#include "composer/composer.h"
#include "unit/unit.h"
#include "io/io.h"
#include <stdio.h>
#include "event/eventqueue_table.h"
#include <pthread.h>

#include "event/servicedef.h"

#include "monitor/tools.h"
#include "composer/proxy_composer.h"




void *wsdproxy_run_composer(void* ptr)
{
	
	COMPOSETHREADDATA_PTR data = (COMPOSETHREADDATA_PTR)ptr;
	
	EVENTQUEUE_TABLE_ENTRY_PTR entry = (EVENTQUEUE_TABLE_ENTRY_PTR)data->entry_event_queue;
	COMPOSER_PTR composer = (COMPOSER_PTR)data->composer;
	
	pthread_mutex_lock(&(entry->mutex));	
	
	
	// Add the identifier of the thread composer to the eventqueue_table 
	pthread_t thread_composer = pthread_self();
	
	
	PROXY_SERVICE_DISCOVER_TABLE services_table;
	proxy_service_discover_table_init(&services_table);

	while (1) 				
	{
		EVENT_PTR evt = eventqueue_table_get_event(entry);
			
		if (evt != NULL)
		{
			fprintf(stdout,"wsdproxy_run_composer: evt is type=%d, src=%p\n", evt->type,evt->stream_id);
			event_print(evt);  
			
			switch (evt->type) 
			{
				case SDP_SERVICE_DESCR_URL_NOPATH:
				case SDP_SERVICE_CTRL_URL_NOPATH:
				{
					char *ip, *url;
					int port;
					parseURL_host_port(evt->value.value_string, &ip, &port, &url);
					fprintf(stdout,"wsdproxy_run_composer: ip=%s, port=%d, url=%s\n", ip, port, url);
					
					SERVICEDEFPTR service_ptr;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						
						servicedef_set_host(service_ptr, ip);
						servicedef_set_port(service_ptr, port);
					}
					else
					{
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						
						servicedef_set_host(service_ptr, ip);
						servicedef_set_port(service_ptr, port);
					}
					//servicedef_set_host(service_ptr, ip);
					//servicedef_set_port(service_ptr, port);
				}
					break;
				
				
				case SDP_SERVICE_METHOD_DESCRIPTION:
				{
					
					SERVICEDEFPTR service_ptr;
					INTERFACEDEFPTR interface_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
					}
					
					
					EVENT_METHOD_DEF_PTR event_methoddef = evt->value.value_void_pointer;
					
				    METHODDEFPTR methoddef = event_methoddef->method;
				    methoddef_print(methoddef);
				    
				    if (!service_is_new_entry)
				    {
				    	if (!servicedef_contains_method(service_ptr, methoddef))
				    	{
				    		interface_ptr = service_ptr->interfacesDefinition;
				    		interfacedef_add_method(interface_ptr, methoddef);
						    servicedef_set_name(service_ptr, event_methoddef->service_name);
						    interfacedef_set_name(interface_ptr, event_methoddef->interface_name);
				    	}
				    }
				    else
				    {
				    	interface_ptr = service_ptr->interfacesDefinition;
				    	interfacedef_add_method(interface_ptr, methoddef);
					    servicedef_set_name(service_ptr, event_methoddef->service_name);
					    interfacedef_set_name(interface_ptr, event_methoddef->interface_name);
				    }
				
		    
				}
				break;

				
				case SCP_RMIOBJ_NUM:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjNum(service_ptr, evt->value.value_longlong);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjNum(service_ptr, evt->value.value_longlong);
					}
					//num = evt->value.value_longlong;
					fprintf(stdout, "SCP_RMIOBJ_NUM value = %llu\n",evt->value.value_longlong);
				}
					break;
				case SCP_RMIOBJ_UNIQUE:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjUnique(service_ptr, evt->value.value_int);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjUnique(service_ptr, evt->value.value_int);
					}
					//unique = evt->value.value_int;
					fprintf(stdout, "SCP_RMIOBJ_UNIQUE value = %d\n",evt->value.value_int);
				}
					break;
				case SCP_RMIOBJ_TIME:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjTime(service_ptr, evt->value.value_longlong);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjTime(service_ptr, evt->value.value_longlong);
					}
					//time = evt->value.value_longlong;
					fprintf(stdout, "SCP_RMIOBJ_TIME value = %llu\n",evt->value.value_longlong);
				}
					break;
				case SCP_RMIOBJ_COUNT:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjCount(service_ptr, evt->value.value_int);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjCount(service_ptr, evt->value.value_int);
					}
					//count = evt->value.value_int;
					fprintf(stdout, "SCP_RMIOBJ_COUNT value = %d\n",evt->value.value_int);
				}
					break;
					
				/*case SCP_RMIOBJ_IP:
				{
					fprintf(stdout, "SCP_RMIOBJ_IP value = %s\n",evt->value.value_string);
				}
					break;
					*/
				case SCP_RMIOBJ_PORT:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjPort(service_ptr, evt->value.value_int);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjPort(service_ptr, evt->value.value_int);
					}
					//port = evt->value.value_int;
					fprintf(stdout, "SCP_RMIOBJ_PORT value = %d\n",evt->value.value_int);
				}
					break;
				
				case SDP_INTERFACE_CTRL_URL_PATH:
				{
					fprintf(stdout, "SDP_INTERFACE_CTRL_URL_PATH value = %d\n",evt->value.value_string);
					
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						INTERFACEDEFPTR interface_ptr = service_ptr->interfacesDefinition;
						interfacedef_set_uripath(interface_ptr, evt->value.value_string);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						
						INTERFACEDEFPTR interface_ptr;
						
						if (service_ptr->interfacesDefinition == NULL)
						{
							interface_ptr = interfacedef_new(service_ptr);
							servicedef_add_interface(service_ptr, interface_ptr);
						}
						else
						{
							interface_ptr = service_ptr->interfacesDefinition;
						}
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						interfacedef_set_uripath(interface_ptr, evt->value.value_string);
					}
				}
				break;
				
				case GENERATE_PROXY:
				{
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						
						SERVICEDEFPTR service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);

						servicedef_print(service_ptr);
						
						
						char* uri_iface_path = service_ptr->interfacesDefinition->uriPath;
						fprintf(stdout, "\nuri_iface_path=%s\n",uri_iface_path);
						
						
						if (uri_iface_path == NULL)
						{
							if (service_ptr->host == NULL)
								servicedef_set_host(service_ptr, get_host_ip());
							if (service_ptr->port == -1)
								servicedef_set_port(service_ptr, get_soapproxy_port());
	
							char url_path[255];
							char url[255];
							sprintf(url_path, "/%s%d/control",  service_ptr->name, service_ptr->indexService);
							sprintf(url, "http://%s:%d%s", get_host_ip(), get_soapproxy_port(), url_path);
							
							composer_publish_event(event_create_string(SDP_SERVICEPROXY_URL_CTRL, url /*"http://192.168.2.2:8080/tmp/device1.xml"*/, evt->stream_id, (void*)evt->src), composer);
							
					    	Table_add_PathServiceDef_Elem( &ServiceDefinitionTable, url_path, service_ptr);
					    	
						    SERVICEDEFPTR srvTmp = Table_get_ServiceDef(ServiceDefinitionTable, url_path);
						    
						    if (srvTmp != NULL)
						    	fprintf(stdout,"\nService %s Found: %s\n",url_path, srvTmp->name);
						    else
						    	fprintf(stdout,"\nService %s NOT Found\n", url_path);
						}
						else
						{
							char url[255];
							sprintf(url,"http://%s:%d%s",service_ptr->host, service_ptr->port, uri_iface_path);
							composer_publish_event(event_create_string(SDP_SERVICEPROXY_URL_CTRL, url /*"http://192.168.2.2:8080/tmp/device1.xml"*/, evt->stream_id, (void*)evt->src), composer);
						}
						/*
					    INTERFACEDEFPTR interface_ptr_tmp;
					    for (interface_ptr_tmp = service_ptr->interfacesDefinition; interface_ptr_tmp != NULL; interface_ptr_tmp=interface_ptr_tmp->next)
					    {
					    	upnpproxy_create_servicedescription(interface_ptr_tmp, service_ptr);
					    }
	*/
						
						
					    /* Adds the service definition to the table that associates servicedefinitions to 
						the related path used in SOAP message calls  in POST of the HTTP HEADER.
	
					       Each RMI service has got only one interface, so the path for the service is:
					       "/"+unique_interface_name+"/control" */
					    
						/*
						INTERFACEDEFPTR iface = service_ptr->interfacesDefinition;
					    if (iface != NULL) {
					    	char path1[255] ="/";
					    	strcat(path1, iface->name);
					    	strcat(path1, "/control");
*/

				//	    }

					}
				}
				    break;

			}
				
		}
		usleep(10);
	}
	
}


void wsdproxy_composer_event_received(EVENT_PTR evt, LISTENER_PTR node)
{
	fprintf(stdout,"WSDPROXY");
	composer_event_received(evt, node);
}

COMPOSER_PTR wsdproxy_composer_create(UNIT_PTR u)
{
	fprintf(stdout, "Creating WSDPROXY composer\n");
		
	COMPOSER_PTR c = composer_create(u);

	c->run_composer = wsdproxy_run_composer;
	c->factory = wsdproxy_composer_create;

	c->func_event_received = wsdproxy_composer_event_received;
	return c;	
}
