#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <io/io.h>
#include <ser/objectparser.h>


static int currentHandle=baseWireHandle;
static j_boolean EndAnnotation;


void parseFieldValue(int fd, SER_FIELDPTR field)
{
	fprintf(stdout,"parseFieldValue() pointer=%p, type=%c name=%s ",field, field->type_code, field->name);
	
    switch (field->type_code) {
    case 'B':
    	parsejByte(fd, &field->value.byte_value);
    	fprintf(stdout," value=%02X \n",field->value.byte_value);
    	break;
    case 'C':
    	parsejByte(fd, &field->value.char_value);
    	fprintf(stdout," value=%c \n",field->value.char_value);
    	break;
    case 'Z':
    	parsejByte(fd, &field->value.boolean_value);
    	fprintf(stdout," value=%d \n",field->value.boolean_value);
    	break;
    case 'I':
    	parsejInt(fd, &field->value.int_value);
    	fprintf(stdout," value=%d \n",field->value.int_value);
    	break;
    case 'J':
    	parsejLong(fd, &field->value.long_value);
    	fprintf(stdout," value=%d \n",field->value.long_value);
    	break;
    case 'S':
    	parsejShort(fd, &field->value.short_value);
    	fprintf(stdout," value=%d \n",field->value.short_value);
    	break; 
    case 'L':  
    case '[':
    	fprintf(stdout," class_name=%s \n",field->class_name);
    	char* tmp = parseObjectOutputStream(fd);
    	break;
    case 'D':
    case 'F':
    	fprintf(stdout,"parseFieldValue() for type code= %c not supported\n",field->type_code);
    	break;   
    default:
       fprintf(stdout,"invalid classDescInfo : %c\n",field->type_code);
    }

}

int parseFieldsValues(int fd, CLASSDESCPTR class_ptr)
{
	fprintf(stdout,"Parse Fields Values for class %s. ", class_ptr->name);
	
	// write fields values
	SER_FIELDPTR fields = class_ptr->field;
	j_short sizeFields = class_ptr->sizeField;
    
	fprintf(stdout,"Field size = %d\n", sizeFields);
	
	int i = 0;    
    for (i = 0; i < sizeFields; i++) {
    	parseFieldValue(fd, fields+i*sizeof(SER_FIELD));
    }
	
    if (class_ptr->superClass != NULL)
    	parseFieldsValues(fd, class_ptr->superClass);
}

OBJECTDESCPTR parseObject(int fd)
{
    OBJECTDESCPTR newPtr = malloc(sizeof(OBJECTDESC));
    
    // parse class description for this object
    CLASSDESCPTR desc = parseClassDescription(fd);
    newPtr->classDescr = desc;

    // parse object description 
    parseObjectData(fd, desc, newPtr);
    
    parseFieldsValues(fd, desc);

    return newPtr;
}

CLASSDESCPTR parseClassDescription(int fd)
{
    j_byte code;
    code=parseCode(fd);
        
    switch (code) {
        case TC_CLASSDESC:
            fprintf(stdout, "CODE : %s = TC_CLASS_DESC\n",toHexString(code));
            return parseClassDesc(fd);

        case TC_NULL:
            fprintf(stdout,"\tTC_NULL \t=>Probably a null super class\n");
            return NULL;

        case TC_PROXYCLASSDESC:
            fprintf(stdout,"TC_PROXYCLASSDESC");
    	    return NULL;
    	    
        default:
			fprintf(stdout, "parseClassDescription. CODE : %s = DON'T UNDERSTAND\n",toHexString(code));

    }
    return NULL;
}

CLASSDESCPTR parseClassDesc(int fd)
{
	int i;
	CLASSDESCPTR newPtr = malloc(sizeof(CLASSDESC));
	char * str=parsejUTF8String(fd);
	for(i = 0; i<strlen(str);i++)
	{
		if(str[i]=='/')
			str[i]='.';
	}
	newPtr->name=str;
	fprintf(stdout,"\tClassName: %s\n",newPtr->name);
	newPtr->id = currentHandle++ -baseWireHandle;
	fprintf(stdout,"\tHandle number: %i\n",(int)newPtr->id);
	
	//Type
		//=> TO DO
	//Parse du serial number
	j_long l;
	parsejLong(fd,&l);
	newPtr->serial=l;
	fprintf(stdout,"\tserial version is : %I64d\n",newPtr->serial);
	j_byte flags;
	parseFlags(fd,&flags);
	newPtr->flag=flags;
	
	
	newPtr->sizeField = parseFields(fd, &(newPtr->field));
	
	//EndAnnotation=false;
	
	char* annotation = parseAnnotation(fd);	
	if (annotation != NULL)
	{
		newPtr->annotation = strdup(annotation);
		fprintf(stdout,"parseClassDesc annotation=%s\n",annotation);
	}
	if(!EndAnnotation)
    {
    	parseCode(fd);
    	fprintf(stdout,"Fin de l'annotation\n");
    	EndAnnotation=false;
    }
    
    newPtr->superClass=parseClassDescription(fd);
    
	return newPtr;
}

void parseFlags(int fd,j_byte *flags)
{
     parsejByte(fd,flags);	
     switch (*flags) {
     case SC_WRITE_METHOD:
		fprintf(stdout,"\tflag= SC_WRITE_METHOD\n");
	    break;
     case SC_SERIALIZABLE:
		fprintf(stdout,"\tflag = SC_SERIALIZABLE\n");
		break;
     case SC_EXTERNALIZABLE:
		fprintf(stdout,"\tflag = SC_EXTERNALIZABLE\n");
		break;
     case (SC_WRITE_METHOD | SC_SERIALIZABLE):
		fprintf(stdout,"\tflag = SC_SERIALIZABLE + SC_WRITE_METHOD\n");
     	break;
     case (SC_BLOCK_DATA | SC_EXTERNALIZABLE):
		fprintf(stdout,"\tflag = SC_EXTERNALIZABLE + SC_BLOCK_DATA\n");
     break;
         
     default:
        fprintf(stdout,"invalid classDescInfo : %c\n", *flags);
     }
 }

/**
 * Fields
 */
int parseFields(int fd, SER_FIELDPTR* fields)
{   
  	j_short count=0;
 	parsejShort(fd,&count);
    fprintf(stdout,"\tNumber of fields: %i\n",(int)count);
   
    /*SER_FIELDPTR fieldPtr*/ *fields = malloc(count*sizeof(SER_FIELD));
    int i;    
    for (i = 0; i < count; i++) {
		parseFieldDesc(fd, (*fields)+(i*sizeof(SER_FIELD)));
    }
    
    fprintf(stdout,"\n\nFIELDS SUMMIRIZE:\n");
    for (i = 0; i < count; i++) {
    	fprintf(stdout,"FIELD %d pointer=%p type=%c name=%s class_name=%s\n",i, ((SER_FIELDPTR)((*fields)+(i*sizeof(SER_FIELD)))),
    			((SER_FIELDPTR)((*fields)+(i*sizeof(SER_FIELD))))->type_code, 
    			((SER_FIELDPTR)((*fields)+(i*sizeof(SER_FIELD))))->name, ((SER_FIELDPTR)((*fields)+(i*sizeof(SER_FIELD))))->class_name);
    }    
    
    
    return count;
}

int parseFieldDesc(int fd, SER_FIELDPTR field)
{
	fprintf(stdout,"parseFieldDesc()\n");
    j_byte typecode;
    char* str;
      
	parsejByte(fd, &typecode);	
	field->type_code = (char)typecode;
	field->class_name = NULL;
	
    switch (typecode) {
    case 'B':
    	str = parsejUTF8String(fd);
		fprintf(stdout,"\tprimitive type code = byte, fieldName = %s\n",str);
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
	    break;
    case 'C':
    	str = parsejUTF8String(fd);
		fprintf(stdout,"\tprimitive type code = char, fieldName = %s\n",str);
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
		break;
    case 'D':
    	str = parsejUTF8String(fd);
		fprintf(stdout,"\tprimitive type code = double, fieldName = %s\n",str);
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
		break;
    case 'F':
    	str = parsejUTF8String(fd);
		fprintf(stdout,"\tprimitive type code = float, fieldName = %s\n",str);
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
		break;
    case 'I':
    	str = parsejUTF8String(fd);
		fprintf(stdout,"\tprimitive type code = integer, fieldName = %s\n",str);
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
		break;
    case 'J':	
    	str = parsejUTF8String(fd);
		fprintf(stdout,"\tprimitive type code = long, fieldName = %s\n",str);
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
		break;
    case 'S':
    	str = parsejUTF8String(fd);
		fprintf(stdout,"\tprimitive type code = short, fieldName = %s\n",str);
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
		break;
    case 'Z':
    	str = parsejUTF8String(fd);
		fprintf(stdout,"\tprimitive type code = boolean, fieldName = %s\n",str);
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
		break;        
    case 'L':
    {
    	str = parsejUTF8String(fd);
		
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
	    // parse String: object type
		
		//char* class_name = (char*)
		char* tmp = parseObjectOutputStream(fd);
		
		
		//if (class_name != NULL)
			//field->class_name = strdup(class_name);
		//else
		field->class_name = malloc(strlen("Ljava/lang/String;")+1);
		strcpy(field->class_name, "Ljava/lang/String;");
			//field->class_name = strdup("Ljava/lang/Stri
		
		fprintf(stdout,"\tObject: type=%c fieldName = %s class_name=%s\n",field->type_code, field->name, field->class_name);
    }	
		break;   
    case '[':
    {
    	str = parsejUTF8String(fd);
	
		//field->name = strdup(str);
		field->name = malloc(strlen(str)+1);
		strcpy(field->name, str);
	    // parse String: array type
		//char* class_name = (char*)
		char* tmp = parseObjectOutputStream(fd);
		
		//if (class_name != NULL)
		//	field->class_name = strdup(class_name);
		//else
		//field->name = strdup(str);
		field->class_name = malloc(strlen("Ljava/lang/String;")+1);
		strcpy(field->class_name, "Ljava/lang/String;");
			//field->class_name = strdup("Ljava/lang/String;");
		
		fprintf(stdout,"\tArray: type=%c fieldName = %s class_name=%s\n",field->type_code, field->name, field->class_name);
    }
		break;   
    default:
       fprintf(stdout,"invalid classDescInfo : %c\n", typecode);
    }
}

char * parseAnnotation(int fd)
{
	if(peekCode(fd)==TC_ENDBLOCKDATA) 
    {
     	parseCode(fd);
        fprintf(stdout,"No Annotation\n");
        EndAnnotation=true;
        return NULL;
    }
    else
    {
    	fprintf(stdout,"parseAnnotation: %x\t\n",peekCode(fd));
    	parseObjectOutputStream(fd);
    	fprintf(stdout,"parseAnnotation: %x\t\n",peekCode(fd));
    	parseObjectOutputStream(fd);
    	char *str;
      //str=(char *)
      OBJECTDESCPTR obj = (OBJECTDESCPTR)parseObjectOutputStream(fd);
      if (obj != NULL && obj->classDescr != NULL && obj->classDescr->field != NULL)
    	  str = (char*)(obj->classDescr->field[0].value.string_value);
      else
    	  str = NULL;
      fprintf(stdout,"Annotation: %s\t\n",str);
      if (str == NULL)
    	  return NULL;
      else
    	  return strdup(str);
    }
    return NULL;
}


void parseClass(int fd)
{
	
	
}


/*
 * TRIED TO IMPLEMENT PARSEARRAY ON EXCEPTION STREAM RECEIVED AS A REPLY
 * OF "RMIHOLDERS.BOOLEAN.SETVALUE_______()" CALL:
 * PROBLEM IN INTERPRETING THE RMI SPECIFICATION FOR ARRAYS
 * I GIVE UP AND WILL TAKE IT UP AGAIN WHEN TRY TO IMPLEMENT PARSING FOR
 * SIMPLE ARRAYS USED AS ARGS IN METHOD CALLS
 * 
 */
void parseArray(int fd)
{
	CLASSDESCPTR desc = parseClassDescription(fd);
	
	j_int size;
	parsejInt(fd, &size);
	fprintf(stdout,"Array size=%d\n",size);
	fflush(stdout);	
	
	//if CLASSDESCR IS OF TYPE "L"
	OBJECTDESCPTR obj_defining_array_element = (OBJECTDESCPTR)parseObjectOutputStream(fd);
	
	j_byte x0;
	parsejByte(fd, &x0);
	fprintf(stdout,"BYTE VALUE array elem 0 = %d\n",x0);
	
	int i;
	for (i = 0; i <size; i++)
	{
		fprintf(stdout,"Array Element %d\n",i);
		
		
		parseFieldsValues(fd, obj_defining_array_element->classDescr);
		
		
		// REPLACES PREVIOUS CODE
		//parseObjectOutputStream(fd);
//		j_byte x1;
//		parsejByte(fd, &x1);
//		fprintf(stdout,"BYTE VALUE array elem 0 = %d\n",x1);
/*
			// parse Fields Velues
			j_int x;
			parsejInt(fd, &x);
			fprintf(stdout,"INT VALUE array elem 0 = %d\n",x);
			parseObjectOutputStream(fd);
			parseObjectOutputStream(fd);
			parseObjectOutputStream(fd);
			
			// Parse 0x73 ????
			j_byte x1;
			parsejByte(fd, &x1);
			fprintf(stdout,"BYTE VALUE array elem 0 = %d\n",x1);
			parsejInt(fd, &x);
			fprintf(stdout,"INT VALUE array elem 0 = %d\n",x);
			parsejByte(fd, &x1);
			fprintf(stdout,"BYTE VALUE array elem 0 = %d\n",x1);
*/

	
	}
	
	int ind_x1;
	j_byte x1;
	for (ind_x1 = 0; ind_x1 < 40; ind_x1++ )
	{
		parsejByte(fd, &x1);
		fprintf(stdout,"\nAfter Element Array1=>%02x",x1&0xff);
	}

}

void parseReference(int fd)
{
	//fprintf(stdout,"parseReference()\n");
	j_int handle=0;
	parsejInt(fd,&handle);
	handle-= (j_int)baseWireHandle;	
	//fprintf(stdout,"\thandle: %i\n",(int)handle);
}

char * parseString(int fd)
{
	
     char * str = parsejUTF8String(fd);
     fprintf(stdout,"parseString(): %s\n",str);
     return str;
}
void* parseObjectOutputStream(int fd)
{	j_byte code;
	
	code=parseCode(fd);
	
	
	
	switch (code) {
		case TC_OBJECT:
				fprintf(stdout, "CODE : %s = TC_OBJECT\n",toHexString(code));
				return parseObject(fd);
			break;
		    case TC_CLASS:
				fprintf(stdout, "CODE : %s = TC_CLASS\n",toHexString(code));
				parseClass(fd);
			break;
		    case TC_ARRAY:
				fprintf(stdout, "CODE : %s = TC_ARRAY\n",toHexString(code));
				parseArray(fd);
			break;
		    case TC_BLOCKDATA:
				fprintf(stdout, "CODE : %s = TC_BLOCKDATA\n",toHexString(code));
		    break;
		    case TC_ENDBLOCKDATA: //Class Annotation ?	
				fprintf(stdout, "CODE : %s = TC_ENDBLOCKDATA\n",toHexString(code));
				return NULL;

		    case TC_REFERENCE: //Reference to an object already parsed		    
				fprintf(stdout, "CODE : %s = TC_REFERENCE\n",toHexString(code));
				parseReference(fd);
				return "NULL";
		    case TC_STRING:
				fprintf(stdout, "CODE : %s = TC_STRING\n",toHexString(code));
				char * str=parseString(fd);
				
				OBJECTDESCPTR newPtr = malloc(sizeof(OBJECTDESC));
    				// parse class description for this object
				newPtr->classDescr = malloc(sizeof(CLASSDESC));
    				newPtr->classDescr->name = strdup("java.lang.String");
				newPtr->classDescr->field = malloc(sizeof(SER_FIELD));
				newPtr->classDescr->field[0].value.string_value = strdup(str);
				
				
				return newPtr;
				// MODIFIED FOR WSD service HelloWorld
				//return str;				
		    case TC_NULL:
				fprintf(stdout, "CODE : %s = TC_NULL\n",toHexString(code));		    	
		    break;
		    default:
				fprintf(stdout, "parseObjectOutputStream. CODE : %s = DON'T UNDERSTAND\n",toHexString(code));
		 }
		 return NULL;
}
/*
 * fd : input file
 * desc : description of the class for the object
 * objDesc : description of the object to be filled in this method
 */
void parseObjectData(int fd, CLASSDESCPTR desc, OBJECTDESCPTR objDesc )
{
	CLASSDESCPTR super;
	fprintf(stdout, "readObjectData() for %s\n",desc->name);
	super = desc->superClass;
	if (super != NULL) {
	    parseObjectData(fd, super, objDesc);
	}
	
	if (((desc->flag)&(SC_WRITE_METHOD))==false) {
		fprintf(stdout, "Normal Read value for %s\n",desc->name);			
	}
	else
	{
		fprintf(stdout, "Parsing customized serialization for %s\n",desc->name);
		char *host, *str=NULL;
		j_int port=0;
		j_short mode=0;
		j_short code=peekCode(fd);
		switch(code)
		{
			case TC_BLOCKDATA:
			{
				parseCode(fd); 
				fprintf(stdout,"\tTC_BLOCKDATA");
				j_byte length=0;
				parsejByte(fd,&length);
				fprintf(stdout,"\tnbBytes= %i\n",length);

				char *dump1, *dump2;
				dump1=parsejUTF8String(fd);
				fprintf(stdout,"\tName Of the Object: %s\n",dump1);
				objDesc->name = dump1;

				dump2=parsejUTF8String(fd); //IP @
				fprintf(stdout,"\t@IP: %s\t",dump2);
				objDesc->ip = dump2;

				parsejInt(fd,&port); //4
				fprintf(stdout,"Port: %i\n",port);
				objDesc->port = port;

				j_long objNum	=0; 
				parsejLong(fd,&objNum); //8
				j_int unique		=0; 
				parsejInt(fd,&unique); //4
				j_long time		=0;
				parsejLong(fd,&time); //8
				j_short count	=0;
				parsejShort(fd,&count); //2
				j_byte bool 		=0;
				parsejByte(fd,&bool); // 1

				fprintf(stdout,"ObjID= ");
				fprintf(stdout,"\tObjUnum: %I64d \n",objNum);
				fprintf(stdout,"\tunique= %i\n",unique);
				fprintf(stdout,"\ttime= %I64d\n",time);
				fprintf(stdout,"\tcount= %i\n",count);
				fprintf(stdout,"\tbool= %i\n",bool);

				objDesc->objNum = objNum;
				objDesc->unique = unique;
				objDesc->time = time;
				objDesc->count = count;
				objDesc->bool = bool;
				break;
			}
		    	
			default: 
				fprintf(stdout, "parseObjectData. CODE : %s = DON'T UNDERSTAND\n",toHexString(code));
				
		}
	}			
		fprintf(stdout, "leave readObjectData() for %s\n",desc->name);
}
