 
#include <netinet/in.h> /* for address structs */
#include <arpa/inet.h>  /* for sockaddr_in */
#include <stdio.h>      /* for printf() and fprintf() */
#include <stdlib.h>     /* for atoi() */
#include <string.h>     /* for strlen() */
#include <unistd.h>     /* for close() */

//#define MAX_LEN  60240   /* maximum receive string size */
#define MIN_PORT 10   /* minimum port allowed */
#define MAX_PORT 65535  /* maximum port allowed */

#include "parser/parser.h"
#include "socket/socket.h"
#include "monitor/tools.h"

void* tcp_socket_listen(void* ptr) {

	SOCKETLISTENTHREADDATA_PTR data = (SOCKETLISTENTHREADDATA_PTR)ptr;
	SOCKET_PTR this_socket = data->socket;
	char* addr_str = data->addr;
	unsigned short port = data->port;
	
	
  int sock;                     /* socket descriptor */
  int flag_on = 1;              /* socket option flag */
  struct sockaddr_in addr;   /* socket address structure */
  char recv_str[MAX_LEN+1];     /* buffer to receive string */
  int recv_len;                 /* length of string received */
  struct sockaddr_in from_addr; /* packet source */
  unsigned int from_len;        /* source addr length */
	int recvSock = 0;

  fprintf(stdout, "TCP listens on address= %s, port= %d.\n",addr_str,port);

  /* create socket to join multicast group on */
  if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) 
	{
    perror("socket() failed");
    exit(1);
  }
  
  /* set reuse port to on to allow multiple binds per host */
  if ((setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &flag_on,sizeof(flag_on))) < 0) 
	{
    perror("setsockopt() failed");
    exit(1);
  }

  /* construct a multicast address structure */
  memset(&addr, 0, sizeof(addr));
  addr.sin_family      = AF_INET;
  addr.sin_addr.s_addr = INADDR_ANY;
  addr.sin_port        = htons(port); 

  /* bind to multicast address to socket */
  if ((bind(sock, (struct sockaddr *) &addr,sizeof(addr))) < 0) 
	{
    perror("bind() failed");
    exit(1);
  }

	if(listen(sock,5) != 0)
	{
    perror("listen() failed");
    exit(1);	
	}
  
	for (;;) 
	{/* loop forever */

    /* clear the receive buffers & structs */
    memset(recv_str, 0, sizeof(recv_str));
    from_len = sizeof(from_addr);
    memset(&from_addr, 0, from_len);

    fprintf(stdout, "TCP is waiting on address= %s, port= %d.\n",addr_str, port);
		
		if((recvSock = accept(sock,(struct sockaddr *) &from_addr, &from_len)) < 0)
		{
			perror("accept() failed");
			exit(1);	
		}		
		
    /* block waiting to receive a packet */
    if ((recv_len = recv(sock, recv_str, MAX_LEN, 0)) < 0) 
		{
      perror("recv failed");
      exit(1);
    }

    /* output received string */
    printf("TCP: Received %d bytes from %s: unsigned=%u int=%d", recv_len,inet_ntoa(from_addr.sin_addr), ntohs(from_addr.sin_port), ntohs(from_addr.sin_port));
					 
    //printf("%s\n", recv_str);
    
    MSG_PTR msg = msg_create(recv_str, recv_len, inet_ntoa(from_addr.sin_addr), ntohs(from_addr.sin_port), 
    				addr_str, port, this_socket);
    
 //   int i;
 //  for (i=0; i<msg->length; i++)
//	     fprintf(stdout, "%c",msg->data[i]&0xff);
			 
 //  fprintf(stdout, "\n\n",msg->length);
    
    socket_publish_msg(msg, this_socket);
    
    
  }

  close(sock);  
}


void* tcp_socket_send(void* ptr)
{
	SOCKETTHREADDATA_PTR data = (SOCKETTHREADDATA_PTR)ptr;
	MSG_PTR msg = data->msg;
	SOCKET_PTR this_socket = data->socket;
	
	struct sockaddr_in addr;
	int fd, cnt;
	struct ip_mreq mreq;
	int flag_on = 1;/* socket option flag */
	
	/* create what looks like an ordinary TCP socket */
	if ((fd=socket(PF_INET,SOCK_STREAM,IPPROTO_TCP)) < 0) 
	{
		perror("socket");
		exit(1);
	}
		
	/* set reuse port to on to allow multiple binds per host */
	if ((setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &flag_on,sizeof(flag_on))) < 0) 
	{
		perror("setsockopt() failed");
		exit(1);
	}
	
	
    // get socket local port
    int localport = socket_get_localport(this_socket); //5230;
    //socket_set_localport(this_socket, localport);
    
    // bind socket to local port 
	struct sockaddr_in local_addr;
	memset(&local_addr, 0, sizeof(local_addr));
	local_addr.sin_family      = AF_INET;
	local_addr.sin_addr.s_addr = htonl(INADDR_ANY);; //inet_addr("192.168.2.2"); //inet_addr(msg->dest_address); //
	local_addr.sin_port        = htons(localport);; //htons(38007/*localport*/); //
	if ((bind(fd, (struct sockaddr *) &local_addr, 
        sizeof(local_addr))) < 0) {
		fprintf(stdout, "tcpsocket_send: binding to local port %d\n", localport);
        perror("tcpsocket_send: bind to local addr failed");
        exit(1);
    } else {
    	fprintf(stdout, "tcpsocket_send: bound to local port %d\n", localport);
    }
	
	
	/* set up destination address */
	memset(&addr,0,sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = inet_addr(msg->dest_address); 
	addr.sin_port = htons(msg->dest_port); 		

	fprintf(stdout, "tcpsocket_send to %s : %d the message:\n",msg->dest_address, msg->dest_port);
	fprintf(stdout, "MESSAGE_LENGTH = %d\n",msg->length);

	if (connect(fd, (struct sockaddr*)&addr, sizeof(addr)) < 0) 
	{
		perror("connect");
		exit(1);
	}
	
	int i;
	for (i=0; i<msg->length; i++)
			fprintf(stdout, "%c",msg->data[i]&0xff,msg->data[i]&0xff);
	fprintf(stdout, "\n\n",msg->length);
	     
	if ((i = send(fd, msg->data, msg->length, 0)) < 0) 
	{
		perror("sendto");
		exit(1);
	}
	
	fprintf(stdout, "MESSAGE SENT: length=%d \n",i);
	
	
	// TCP does not allow SO_REUSEADDR and SO_REUSEPORT:
	// the connection to the server must be kept by the client that 
	// sends the request and the reply is received on the same connected socket
	// that sends the request
	  char recv_str[MAX_LEN+1]; 
	  int recv_len; 
	  struct sockaddr_in from_addr; /* packet source */
	  unsigned int from_len;        /* source addr length */
	  
    /* block waiting to receive a packet 
    if ((recv_len = recvfrom(fd, recv_str, MAX_LEN, 0, 
            (struct sockaddr*)&from_addr, &from_len)) < 0)
	{
      perror("recv failed");
      exit(1);
    }
	/*/
	recv_len = 0;
	int bytes_recv = 0;
	char recv_str_tmp[MAX_LEN+1]; 
	while ((recv_len <1000) && (bytes_recv = recvfrom(fd, recv_str_tmp, MAX_LEN, 0, 
	            (struct sockaddr*)&from_addr, &from_len)))
	{
		int j;
		for (j=0; j<bytes_recv; j++)
			recv_str[recv_len+j] = recv_str_tmp[j];
		recv_len += bytes_recv;
		
	    printf("\nTCP: Received %d bytes from %s: unsigned=%u int=%d", recv_len, 
	            inet_ntoa(from_addr.sin_addr), ntohs(from_addr.sin_port), ntohs(from_addr.sin_port));
    }
	  /**/  
    close(fd);  
    
	  printf("TCP: Received %d bytes \n" , recv_len);
	    for (i=0; i<300; i++)
		     fprintf(stdout, "%c",recv_str[i]&0xff);
	   
	    
    /* output received string */
    printf("\nTCP: Received %d bytes from %s: unsigned=%u int=%d", recv_len, 
            inet_ntoa(from_addr.sin_addr), ntohs(from_addr.sin_port), ntohs(from_addr.sin_port));
    
    MSG_PTR msg_reply = msg_create(recv_str, recv_len, inet_ntoa(from_addr.sin_addr), ntohs(from_addr.sin_port),
    		get_host_ip()/*"192.168.2.2"*/, localport, this_socket);
    

 //   for (i=0; i<msg_reply->length; i++)
//	     fprintf(stdout, "%c",msg_reply->data[i]&0xff);
			 
 //  fprintf(stdout, "\n\n",msg_reply->length);
    
    socket_publish_msg(msg_reply, this_socket);

	
	
	
}

SOCKET_PTR tcp_socket_create(UNIT_PTR u)
{
	fprintf(stdout, "Creating tcp socket\n");
	
	SOCKET_PTR s = (SOCKET_PTR)malloc(sizeof(SOCKET));
	s->func_msg_received = socket_msg_received;
	s->func_publish_msg = socket_publish_msg;
	s->msg_listeners = NULL;
	s->unit = u;	
	
	s->func_send = tcp_socket_send;
	s->factory = tcp_socket_create;

	s->func_listen = tcp_socket_listen;
	
	return s;	
}



