#include <netinet/in.h> /* for address structs */
#include <arpa/inet.h>  /* for sockaddr_in */
#include <stdio.h>      /* for printf() and fprintf() */
#include <stdlib.h>     /* for atoi() */
#include <string.h>     /* for strlen() */
#include <unistd.h>     /* for close() */

//#define MAX_LEN  1024   /* maximum receive string size */
#define MIN_PORT 10   /* minimum port allowed */
#define MAX_PORT 65535  /* maximum port allowed */

#include "parser/parser.h"
#include "socket/socket.h"
#include "monitor/monitor.h"
#include "monitor/tools.h"

void* udpmc_socket_listen(void* ptr /*SOCKET_PTR this_socket, char* mc_addr_str, unsigned short mc_port */) {

	SOCKETLISTENTHREADDATA_PTR data = (SOCKETLISTENTHREADDATA_PTR)ptr;
	SOCKET_PTR this_socket = data->socket;
	char* mc_addr_str = data->addr;
	unsigned short mc_port = data->port;

	
	
  int sock;                     /* socket descriptor */
  int flag_on = 1;              /* socket option flag */
  struct sockaddr_in mc_addr;   /* socket address structure */
  char recv_str[MAX_LEN+1];     /* buffer to receive string */
  int recv_len;                 /* length of string received */
  struct ip_mreq mc_req;        /* multicast request structure */
  //char* mc_addr_str;            /* multicast IP address */
  //unsigned short mc_port;       /* multicast port */
  struct sockaddr_in from_addr; /* packet source */
  unsigned int from_len;        /* source addr length */

  /* validate number of arguments 
  if (argc != 3) {
    fprintf(stdout, 
            "Usage: %s <Multicast IP> <Multicast Port>\n", 
            argv[0]);
    exit(1);
  }

  mc_addr_str = argv[1];       arg 1: multicast ip address 
  mc_port = atoi(argv[2]);     arg 2: multicast port number 
*/

  fprintf(stdout, "UDPMulticast listens on address= %s, port= %d.\n",mc_addr_str,mc_port);

  /* validate the port range */
  if ((mc_port < MIN_PORT) || (mc_port > MAX_PORT)) {
    fprintf(stdout, "Invalid port number argument %d.\n",
            mc_port);
    fprintf(stdout, "Valid range is between %d and %d.\n",
            MIN_PORT, MAX_PORT);
    exit(1);
  }

  /* create socket to join multicast group on */
  if ((sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
    perror("socket() failed");
    exit(1);
  }
  
  /* set reuse port to on to allow multiple binds per host */
  if ((setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &flag_on,
       sizeof(flag_on))) < 0) {
    perror("setsockopt() failed");
    exit(1);
  }

  /* construct a multicast address structure */
  memset(&mc_addr, 0, sizeof(mc_addr));
  mc_addr.sin_family      = AF_INET;
  mc_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  mc_addr.sin_port        = htons(mc_port);

  /* bind to multicast address to socket */
  if ((bind(sock, (struct sockaddr *) &mc_addr, 
       sizeof(mc_addr))) < 0) {
    perror("bind() failed");
    exit(1);
  }

  /* construct an IGMP join request structure */
  mc_req.imr_multiaddr.s_addr = inet_addr(mc_addr_str);
  mc_req.imr_interface.s_addr = htonl(INADDR_ANY);

  /* send an ADD MEMBERSHIP message via setsockopt */
  if ((setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, 
       (void*) &mc_req, sizeof(mc_req))) < 0) {
    perror("setsockopt() failed");
    exit(1);
  }

  for (;;) {          /* loop forever */

    /* clear the receive buffers & structs */
    memset(recv_str, 0, sizeof(recv_str));
    from_len = sizeof(from_addr);
    memset(&from_addr, 0, from_len);

    fprintf(stdout, "UDPMulticast is waiting on address= %s, port= %d.\n",mc_addr_str,mc_port);
    /* block waiting to receive a packet */
    if ((recv_len = recvfrom(sock, recv_str, MAX_LEN, 0, 
         (struct sockaddr*)&from_addr, &from_len)) < 0) {
      perror("recvfrom() failed");
      //exit(1);
    }

    /* output received string */
    printf("UDPMulticast Received %d bytes from %s: unsigned=%u int=%d", recv_len, 
           inet_ntoa(from_addr.sin_addr), ntohs(from_addr.sin_port), ntohs(from_addr.sin_port));
    //printf("%s\n", recv_str);
    
    
    if (!avoidlooptable_containsentry(inet_ntoa(from_addr.sin_addr), ntohs(from_addr.sin_port), mc_addr_str,mc_port))
    {
		        MSG_PTR msg = msg_create(recv_str, recv_len, inet_ntoa(from_addr.sin_addr), ntohs(from_addr.sin_port), 
						mc_addr_str, mc_port, this_socket);
		
		 int i;
		   for (i=0; i<msg->length; i++)
		     fprintf(stdout, "%c",msg->data[i]&0xff);
		  fprintf(stdout, "\n\n",msg->length);
		
		
		socket_publish_msg(msg, this_socket);
		
		//PARSER_PTR parser = slp_parser_create(NULL);
		//slp_parse_msg(parser, msg->data, msg->length);
		//slp_parse_msg(parser, recv_str, recv_len);
    }
    else
    	fprintf(stdout,"\navoidlooptable_containsentry (%s,%d - %s,%d)\n",inet_ntoa(from_addr.sin_addr), ntohs(from_addr.sin_port), mc_addr_str,mc_port);
    

    
  }

  /* send a DROP MEMBERSHIP message via setsockopt */
  if ((setsockopt(sock, IPPROTO_IP, IP_DROP_MEMBERSHIP, 
       (void*) &mc_req, sizeof(mc_req))) < 0) {
    perror("setsockopt() failed");
    exit(1);
  }

  close(sock);  
}


void* udpmcsocket_send(void* ptr)
{
	SOCKETTHREADDATA_PTR data = (SOCKETTHREADDATA_PTR)ptr;
	MSG_PTR msg = data->msg;
	SOCKET_PTR this_socket = data->socket;
	
	struct sockaddr_in addr;
    int fd, cnt;
    struct ip_mreq mreq;
    int flag_on = 1;              /* socket option flag */ 
    
    /* create what looks like an ordinary UDP socket */
    if ((fd=socket(AF_INET,SOCK_DGRAM,0)) < 0) {
    	perror("socket");
    	//  exit(1);
     }
     	
    /* set reuse port to on to allow multiple binds per host */
    if ((setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &flag_on,
         sizeof(flag_on))) < 0) {
      perror("setsockopt() failed");
      exit(1);
    }
    
    
    // get socket local port
    int localport = socket_get_localport(this_socket); //5230;
    //socket_set_localport(this_socket, localport);
    
    // bind socket to local port 
	struct sockaddr_in local_addr;
	memset(&local_addr, 0, sizeof(local_addr));
	local_addr.sin_family      = AF_INET;
	local_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	local_addr.sin_port        = htons(localport); //htons(localport);
	if ((bind(fd, (struct sockaddr *) &local_addr, 
        sizeof(local_addr))) < 0) {
        perror("udpmcsocket_send: bind to local addr failed");
        exit(1);
    } else {
    	fprintf(stdout, "udpmcsocket_send: bound to local port %d\n", localport);
    }

	int optarg = 255;
	if(setsockopt(fd, IPPROTO_IP, IP_MULTICAST_TTL, &optarg,sizeof(optarg)))
    {
        return ;
    }

	
	
	
    /* set up destination address */
    memset(&addr,0,sizeof(addr));
    addr.sin_family=AF_INET;
    addr.sin_addr.s_addr=inet_addr(msg->dest_address);
    addr.sin_port=htons(msg->dest_port);
     

    fprintf(stdout, "udpmcsocket_send to %s : %d the message:\n",msg->dest_address, msg->dest_port);
    fprintf(stdout, "MESSAGE_LENGTH = %d\n",msg->length);
    int i;
    for (i=0; i<msg->length; i++)
	     fprintf(stdout, "%c",msg->data[i]&0xff);
    fprintf(stdout, "\n\n",msg->length);
	     
    
    /* 
     * TO AVOID MESSAGE LOOPS
     */
    avoidlooptable_addentry(get_host_ip(), localport, msg->dest_address, msg->dest_port);

    
	if (sendto(fd, msg->data, msg->length, 0, (struct sockaddr *) &addr,
	     sizeof(addr)) < 0) {
	       perror("sendto");
	       exit(1);
	}

	
	
	close(fd);  

}

SOCKET_PTR udpmcsocket_create(UNIT_PTR u)
{
	fprintf(stdout, "Creting udp mc socket\n");
	
	SOCKET_PTR s = (SOCKET_PTR)malloc(sizeof(SOCKET));
	s->func_msg_received = socket_msg_received;
	s->func_publish_msg = socket_publish_msg;
	s->msg_listeners = NULL;
	s->unit = u;	
	
	s->func_send = udpmcsocket_send;
	s->factory = udpmcsocket_create;
	
	s->func_listen = udpmc_socket_listen;
	//socket_add_msglistener_unit(s, u);
	
	/*
	char mc_addr_str[255];
	strcpy(mc_addr_str, "239.255.255.253");
	unsigned short mc_port = 427;
	socket_listen(s, mc_addr_str, mc_port );
	*/
	
	return s;	
}



