#include "socket/socket.h"
#include "unit/unit.h"

#include <stdio.h>
#include <pthread.h>



void socket_listen(SOCKET_PTR socket, char* addr, unsigned short port ) 
{
	fprintf(stdout, "socket_listen addr=%s port=%d \n", addr,port); 
	pthread_t socket_listen_thread;
	SOCKETLISTENTHREADDATA_PTR data = malloc(sizeof(SOCKETLISTENTHREADDATA));
	data->socket = socket;
	data->addr = strdup(addr);
	data->port = port;
	fprintf(stdout, "socket_listen (1) addr=%s port=%d \n", addr,data->port); 
	pthread_create(&socket_listen_thread, NULL, socket->func_listen, (void*)data);
	fprintf(stdout, "socket_listen (2) addr=%s port=%d \n", addr,data->port); 
}

void socket_msg_received(MSG_PTR msg, LISTENER_PTR node)
{
	fprintf(stdout, "socket receives msg length=%d src_addr=%s src_port=%d \n", msg->length, msg->src_address, msg->src_port);
	
	SOCKET_PTR s = (SOCKET_PTR)node->listener.socket;
	
	pthread_t send_thread;
	
	SOCKETTHREADDATA_PTR data = malloc(sizeof(SOCKETTHREADDATA));
	data->msg = msg;
	data->socket = s;
	
	fprintf(stdout, "\nCREATING PTHREAD socket = %p\n", send_thread);
	
	pthread_create(&send_thread, NULL, s->func_send, (void*)data);
}

int socket_add_msglistener(SOCKET_PTR socket, struct listener* evt_list)
{
	fprintf(stdout, "socket_add_msglistener\n");
        LISTENER_LIST_ENTRY_PTR list_entry = (LISTENER_LIST_ENTRY_PTR)malloc(sizeof(LISTENER_LIST_ENTRY));
	list_entry->node = evt_list;
	list_entry->next = NULL;
	
	if (socket->msg_listeners == NULL) {
		socket->msg_listeners = list_entry;	
	} else {
		list_entry->next = socket->msg_listeners;
		socket->msg_listeners = list_entry;
	}
}
	
int socket_add_msglistener_unit(SOCKET_PTR socket, struct unit* u)
{
	fprintf(stdout, "socket_add_msglistener_unit\n");
	LISTENER_PTR evt_list = (LISTENER_PTR)malloc(sizeof(LISTENER_PTR));
	evt_list->type = LISTENER_UNIT;
	evt_list->listener.unit = u;
	socket_add_msglistener(socket, evt_list);
}

int socket_add_msglistener_parser(SOCKET_PTR socket, struct parser* p)
{
	fprintf(stdout, "socket_add_msglistener_parser\n");
	LISTENER_PTR evt_list = (LISTENER_PTR)malloc(sizeof(LISTENER_PTR));
	evt_list->type = LISTENER_PARSER;
	evt_list->listener.parser = p;
	socket_add_msglistener(socket, evt_list);
}

int socket_add_msglistener_monitor(SOCKET_PTR socket, struct monitor* m)
{
	fprintf(stdout, "socket_add_msglistener_monitor\n");
	LISTENER_PTR evt_list = (LISTENER_PTR)malloc(sizeof(LISTENER_PTR));
	evt_list->type = LISTENER_MONITOR;
	evt_list->listener.monitor = m;
	socket_add_msglistener(socket, evt_list);
}

void socket_publish_msg(MSG_PTR msg, SOCKET_PTR socket)
{
	fprintf(stdout, "socket_publish_msg\n");
	LISTENER_LIST_PTR l;

	for (l = socket->msg_listeners; l != NULL; l = l->next)
	{
	   LISTENER_PTR node = l->node;

	   if (node->type == LISTENER_PARSER) {
		PARSER_PTR p = (PARSER_PTR)node->listener.parser;
		fprintf(stdout, "socket_publish_msg on parser %p\n",p);
		
			p->func_msg_received(msg, node);
	   }
	   /*else if (node->type == LISTENER_COMPOSER) {
		COMPOSER_PTR p = (COMPOSER_PTR)node->listener.composer;
		fprintf(stdout, "socket_publish_msg on composer %p\n",p);
		
			p->func_msg_received(msg, node);
	   }*/
	   else if (node->type == LISTENER_UNIT) {
		UNIT_PTR u = (UNIT_PTR)node->listener.unit;
		fprintf(stdout, "socket_publish_msg on unit %p\n",u);
		
			u->func_msg_received(msg, node);
	   }
  	   else if (node->type == LISTENER_MONITOR) {
		MONITOR_PTR m = (MONITOR_PTR)node->listener.monitor;
		fprintf(stdout, "socket_publish_msg on monitor %p\n",m);
		
			m->func_msg_received(msg, node);
	   }
	}
}


SOCKET_PTR socket_create_without_unit()
{
	fprintf(stdout, "Creting socket\n");
		
	SOCKET_PTR s = (SOCKET_PTR)malloc(sizeof(SOCKET));
	s->func_msg_received = socket_msg_received;
	s->func_publish_msg = socket_publish_msg;
	//s->listeners = NULL;
	s->msg_listeners = NULL;
	s->unit = NULL;	
	
	return s;	
}

SOCKET_PTR socket_create(UNIT_PTR u)
{
	SOCKET_PTR s = socket_create_without_unit();
	s->unit = u;	
	
	return s;	
}

int socket_get_localport(SOCKET_PTR socket)
{
	return socket->local_port;
}

int socket_set_localport(SOCKET_PTR socket, int local_port)
{
	socket->local_port = local_port;	
}

