#include "unit/unit.h"
#include <stdio.h>

#include "socket/socket.h"
#include "event/eventqueue_table.h"
#include "event/event.h"




void upnpunit_event_received(EVENT_PTR evt, LISTENER_PTR node)
{
	fprintf(stdout, "UPNP");
	unit_event_received(evt, node);
}


UNIT_PTR upnpunit_create(char* sm_filename)
{
	
	UNIT_PTR u = unit_create(sm_filename, NULL);
	fprintf(stdout, "Creating UPnP unit: %p\n", u);
	
	PARSER_PTR p = ssdp_parser_create(u);
	char key[255];
	strcpy(key, "SSDP");
	unit_addparser(u, key, p);
	
	/*
	 *  added to parse msg received by monitor
	 *  and to dispatch  events generated by parser to the unit
	 */
	unit_add_msglistener_parser(u, p);
	parser_add_evtlistener_unit(p, u);
	

	PARSER_PTR p2 = upnp_devicedescr_parser_create(u);
	char key5[255];
	strcpy(key5, "UPNP_DEVICEDESCR");
	unit_addparser(u, key5, p2);

	PARSER_PTR p3 = upnp_servicedescr_parser_create(u);
	char key7[255];
	strcpy(key7, "UPNP_SERVICEDESCR");
	unit_addparser(u, key7, p3);
	
	//unit_add_msglistener_parser(u, p);
 	//u->current_parser = p;
	//parser_add_evtlistener_unit(p, u);
	
	COMPOSER_PTR c = ssdp_composer_create(u);
	char key1[255];
	strcpy(key1, "SSDP");
	unit_addcomposer(u, key1, c);

	COMPOSER_PTR c2 = upnp_devicedescr_composer_create(u);
	char key4[255];
	strcpy(key4, "UPNP_DEVICEDESCR");
	unit_addcomposer(u, key4, c2);
	
	COMPOSER_PTR c3 = upnp_servicedescr_composer_create(u);
	char key8[255];
	strcpy(key8, "UPNP_SERVICEDESCR");
	unit_addcomposer(u, key8, c3);
	
//	unit_add_evtlistener_composer(u, c);
// 	u->current_composer = c;
	
	SOCKET_PTR s = udpmcsocket_create(u);
	char key2[255];
	strcpy(key2, "UDPMC");
	unit_addsocket(u, key2, s);	

	SOCKET_PTR s2 = udp_socket_create(u);
	char key3[255];
	strcpy(key3, "UDP");
	unit_addsocket(u, key3, s2);	

	SOCKET_PTR s3 = tcp_socket_create(u);
	char key6[255];
	strcpy(key6, "TCP");
	unit_addsocket(u, key6, s3);	
	
//	socket_add_msglistener_unit(s, u);
//	unit_add_msglistener_socket(u, s);
	
	// how can I configure the socket?
	//char mc_addr_str[255];
	//strcpy(mc_addr_str, "239.255.255.253");
	//unsigned short mc_port = 427;
	
	
	 u->func_event_received = upnpunit_event_received; 
	
	 
		/*
		 *	to be moved in monitor and find a generic solution
		 *	for unit creation with factory and set as listeners
		 *	on unit of the other and the other way round 
		 */
		 UNIT_PTR slp_unit = slpunit_create2("slpunit.sm");
		 unit_add_evtlistener_unit(u, slp_unit);
		 unit_add_evtlistener_unit(slp_unit, u);
		 
		 UNIT_PTR wsd_unit = wsdunit_create2("wsdunit.sm");
		 unit_add_evtlistener_unit(u, wsd_unit);
		 unit_add_evtlistener_unit(wsd_unit, u);
		 
		 UNIT_PTR upnpproxy_unit = upnpproxy_unit_create2("upnpproxy_unit.sm");
		 unit_add_evtlistener_unit_proxy(u, upnpproxy_unit );
		 unit_add_evtlistener_unit(upnpproxy_unit , u);
		 
		 
	return u;	
}

UNIT_PTR upnpunit_create2(char* sm_filename)
{
	
	UNIT_PTR u = unit_create(sm_filename, NULL);
	fprintf(stdout, "Creating UPnP unit: %p\n", u);
	
	PARSER_PTR p = ssdp_parser_create(u);
	char key[255];
	strcpy(key, "SSDP");
	unit_addparser(u, key, p);
	
	

	PARSER_PTR p2 = upnp_devicedescr_parser_create(u);
	char key5[255];
	strcpy(key5, "UPNP_DEVICEDESCR");
	unit_addparser(u, key5, p2);

	PARSER_PTR p3 = upnp_servicedescr_parser_create(u);
	char key7[255];
	strcpy(key7, "UPNP_SERVICEDESCR");
	unit_addparser(u, key7, p3);
	
	//unit_add_msglistener_parser(u, p);
 	//u->current_parser = p;
	//parser_add_evtlistener_unit(p, u);
	
	COMPOSER_PTR c = ssdp_composer_create(u);
	char key1[255];
	strcpy(key1, "SSDP");
	unit_addcomposer(u, key1, c);

	COMPOSER_PTR c2 = upnp_devicedescr_composer_create(u);
	char key4[255];
	strcpy(key4, "UPNP_DEVICEDESCR");
	unit_addcomposer(u, key4, c2);
	
	COMPOSER_PTR c3 = upnp_servicedescr_composer_create(u);
	char key8[255];
	strcpy(key8, "UPNP_SERVICEDESCR");
	unit_addcomposer(u, key8, c3);
	
//	unit_add_evtlistener_composer(u, c);
// 	u->current_composer = c;
	
	SOCKET_PTR s = udpmcsocket_create(u);
	char key2[255];
	strcpy(key2, "UDPMC");
	unit_addsocket(u, key2, s);	

	SOCKET_PTR s2 = udp_socket_create(u);
	char key3[255];
	strcpy(key3, "UDP");
	unit_addsocket(u, key3, s2);	

	SOCKET_PTR s3 = http_socket_create(u); //tcp_socket_create(u);
	char key6[255];
	strcpy(key6, /*"TCP"*/ "HTTP");
	unit_addsocket(u, key6, s3);	
	
//	socket_add_msglistener_unit(s, u);
//	unit_add_msglistener_socket(u, s);
	
	// how can I configure the socket?
	//char mc_addr_str[255];
	//strcpy(mc_addr_str, "239.255.255.253");
	//unsigned short mc_port = 427;
	
	
	 u->func_event_received = upnpunit_event_received; 

	
	return u;	
}

