#include <vm/jclass.h>
#include <vm/opcode.h>
#include <vm/attributes.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>


void code_add_ref_instr(ATTR_PTR attr,uint8 op,TAG t,const char* className,const char* varName, const char* type)
{
	POOL_PTR pool=attr->pool;
	uint16* index=add_rec_ref_entry(pool,t,className,varName,type);
	code_new_x1_instr(attr,op,(uint8*)index);
}

extern	void code_add_field_instr(ATTR_PTR attrCode,uint8 opcode,const char* className,const char* varName, const char* type)
{
	code_add_ref_instr(attrCode,opcode,CONSTANT_Fieldref,className,varName,type);	
}

void code_add_method_instr(ATTR_PTR attr,uint8 opcode,const char* returnType,const char*methodName,const char* signature)
{
	switch(opcode)
	{
		case OP_INVOKEINTERFACE:
			code_add_ref_instr(attr,opcode,CONSTANT_InterfaceMethodref,returnType,methodName,signature);
		break;
		default:
			code_add_ref_instr(attr,opcode,CONSTANT_Methodref,returnType,methodName,signature);
	}
}

void code_add_type_instr(ATTR_PTR attr,uint8 op,const char* type)
{
	POOL_PTR pool=attr->pool;
	uint16* index=add_rec_class_entry(pool,type);
	code_new_x1_instr(attr,op,(uint8*)index);
}

void code_new_x0_instr(ATTR_PTR attr,uint8 op)
{
	ATTR_CODE_PTR code=attr->info.code;
	INSTR_PTR instr=malloc(sizeof(INSTR));
	instr->opcode=op;
	instr->length	=(uint8)atoi(opcode[op][1]);
	instr->pc=code->current_pc; 
	instr->mnem		=opcode[op][2];
	instr->index	=(void*)0;
	list_append(code->listCode,lnode_create(instr));
	code->current_pc+=instr->length; 
}

INSTR_PTR code_new_x1_instr(ATTR_PTR attr,uint8 op,uint8* index)
{
	ATTR_CODE_PTR code=attr->info.code;
	INSTR_PTR instr=malloc(sizeof(INSTR));
	instr->opcode=op;
	instr->length	=(uint8)atoi(opcode[op][1]);
	instr->pc=code->current_pc; 
	instr->mnem		=opcode[op][2];
	instr->index	=index;
	list_append(code->listCode,lnode_create(instr));
	code->current_pc+=instr->length;
	return instr;
}

INSTR_PTR code_new_x2_instr(ATTR_PTR attr,uint8 op,uint8* index)
{

}

INSTR_PTR code_add_jump_instr_index(ATTR_PTR attr,uint8 opcode,uint16 jump)
{
	ATTR_CODE_PTR code=attr->info.code;
	uint16* pc=malloc(sizeof(uint8)*2);
	*pc=jump;
	//*pc=code->current_pc+jump; //The jvm does this herself
	return code_new_x1_instr(attr,opcode,(uint8*)pc);	
}


/***
* NEED TO BE DEBUG
* ERROR WHEN THERE ARE SEVERAL GOTO TO UNDIFINED LABELS
**/
INSTR_PTR code_add_jump_instr(ATTR_PTR attr,uint8 opcode,char* label)
{
	hash_t* hash=attr->info.code->hashLabel;
	hnode_t* node=hash_lookup(hash,label);
	LABEL_PTR l;
	uint8* pc;
	if(node!=NULL)
	{
		l=(LABEL_PTR)hnode_get(node);
		
		if(l->pc==0)
		{
			pc=&l->pc;
		}
		else
		{
			pc=malloc(sizeof(uint8)*2);
			*(uint16*)pc=l->pc;
		}
		INSTR_PTR instr=code_new_x1_instr(attr,opcode,pc);
		return instr;
	}else
	{
		INSTR_PTR instr=code_new_x1_instr(attr,opcode,(void*)0);
		hnode_t* node=hnode_create(instr);
		hash_insert(hash,node,label);
		return instr;
	}
}

void code_add_label(ATTR_PTR attr, char* label)
{
	ATTR_CODE_PTR code=attr->info.code;
	hash_t* hash=attr->info.code->hashLabel;
	hnode_t* node=hash_lookup(hash,label);
	INSTR_PTR instr;
	LABEL_PTR l;
	l=malloc(sizeof(LABEL));
	l->label=label;
	l->pc=code->current_pc;
	if(node!=NULL)
	{
		instr=(INSTR_PTR)hnode_get(node);
		assert(instr->index==NULL);
		instr->index=malloc(sizeof(uint8)*2);
		*(uint16*)instr->index=l->pc;
		hnode_put(node,l);
	}else
	{
		node=hnode_create(l);
		hash_insert(hash,node,label);
	}
}

void code_add_try_catch(ATTR_PTR attr,const char* start,const char* end,const char* handler,const char* exception)
{
	LABEL_PTR l;
	hnode_t* node;
	hash_t* hash=attr->info.code->hashLabel;
	list_t* list=attr->info.code->listException;
	EXCEPTION_PTR ex=malloc(sizeof(EXCEPTION));	
	node=hash_lookup(hash,start);
	assert(node!=NULL);
	l=(LABEL_PTR)hnode_get(node);
	ex->start_pc=l->pc;
	node=hash_lookup(hash,end);
	assert(node!=NULL);
	l=(LABEL_PTR)hnode_get(node);
	ex->end_pc=l->pc;
	node=hash_lookup(hash,handler);
	assert(node!=NULL);
	l=(LABEL_PTR)hnode_get(node);
	ex->handler=l->pc;
	list_append(list,lnode_create(ex));
	ex->index=add_rec_class_entry(attr->pool,exception);
}

/**
*
*
*/
void code_add_free_labels(ATTR_PTR attr,char* l)
{
	ATTR_CODE_PTR code=attr->info.code;
	hash_t* hash=code->hashLabel;
	hnode_t* node=hash_lookup(hash,l);
	hash_delete(hash,node);
	LABEL_PTR lptr=(LABEL_PTR)hnode_get(node);
	free(lptr); //lptr->label?
	hnode_destroy(node);
}

/**
*
*/
void code_add_ldc(ATTR_PTR attr,uint8 type,void* data)
{
	POOL_PTR pool=attr->pool;
	list_t* list;
	uint16* index;
	switch(type)
	{
		case CONSTANT_Long:	
		{
			j_long l=*(j_long*)data;
			index=add_rec_long_entry(pool,l);
			code_new_x1_instr(attr,OP_LDC2_W,(uint8*)index);
			break;
		}
		case CONSTANT_Double:
		{
			printf("LDC DOUBLE: NOT IMPLMENTED ");
			break;
		}			
		case CONSTANT_Integer:
		{
			printf("LDC INTEGER: NOT IMPLMENTED ");
			break;
		}			
		case CONSTANT_Float:
		{
			printf("LDC FLOAT: NOT IMPLMENTED ");
			break;
		}			
		case CONSTANT_String:
		{
			char* str=(char*)data;
			index=add_rec_string_entry(pool,str);
			code_new_x1_instr(attr,OP_LDC,(uint8*)index);
			break;
		}
		default:
		{
			printf("LDC ?: NOT UNDERSTOOD");	
		}
	}
}

/**
*
*/
void code_add_single_instr(ATTR_PTR attr,uint8 op)
{
	code_new_x0_instr(attr,op);
}

void code_set_max(ATTR_PTR attr,uint8 stack,uint8 local)
{
	ATTR_CODE_PTR code=attr->info.code;
	code->maxStack=stack;
	code->maxLocal=local;
}

/**
*
*/
void method_add_exceptions(ATTR_PTR attr,char * name)
{
	ATTR_EXCEPTION_PTR ex=attr->info.exception;
	ex->nb++;
	ex->tab=realloc(ex->tab,ex->nb*sizeof(uint16));
	ex->tab[ex->nb-1]=add_rec_class_entry(attr->pool,name);
	attr->attr_length=ex->nb*sizeof(uint16)+2;
}

void attribute_add_constant_value(ATTR_PTR attr, TAG t, const void* data)
{
	ATTR_CONSTANT_PTR c=attr->info.constant;
	uint32 length;
	switch(t)
	{
		case CONSTANT_Long:
			c->constant_value_index=add_rec_long_entry(attr->pool,*((j_long*)data));
			length=sizeof(j_long);
			break;
		/*case CONSTANT_Double:
			c->constant_value_index=add_rec_double_entry(attr->pool,(j_double*)data);
			break;
		case CONSTANT_Float:
			c->constant_value_index=add_rec_float_entry(attr->pool,(j_float*)data);
			break;
		case CONSTANT_Integer:
			c->constant_value_index=add_rec_integer_entry(attr->pool,(j_int*)data);
			break;
		case CONSTANT_String:
			c->constant_value_index=add_rec_string_entry(attr->pool,(char*)data);
			break;*/
		default:
		{
			printf("attribute_add_constant_value(%s) not implemented",tagtable[t]);
		}
	}
}
/**
*
*/
void code_print(ATTR_PTR attr)
{
	ATTR_CODE_PTR code=attr->info.code;
	INSTR_PTR instr;
	list_t* list=code->listCode;
	lnode_t* node;
	uint16* i1;
	uint8*  i2;
	pool_sort_constant_pool(attr->pool);
	for(node = list_first(list); node != 0; node = list_next(list, node))
	{
		instr=(INSTR_PTR)lnode_get(node);	
		fprintf(stdout,"pc:%i \t s:%i >\t%s",instr->pc,instr->length,instr->mnem);
		switch(instr->length)
		{	case 3:
				{
					fprintf(stdout,"index: %i\n",*(uint16*)instr->index);
					break;
				}
			case 2:
				fprintf(stdout,"index: %i\n",*instr->index);
				break;
			case 1:
				
			default:		
			fprintf(stdout,"\n");
		}
	}
}

/**
*
*/
void entry_print(POOL_PTR pool, uint16 index)
{
	POOL_ENTRY_PTR entry=pool->tab[index];
}

/**
* 
*/
void code_print_try_catch(ATTR_PTR attr)
{

	ATTR_CODE_PTR code=attr->info.code;
	EXCEPTION_PTR ex;
	list_t* list=code->listException;
	lnode_t* node;	
	for(node = list_first(list); node != 0; node = list_next(list, node))
	{
		ex=(EXCEPTION_PTR)lnode_get(node);	
		fprintf(stdout,"TryCatchBlock\ts:%i\te:%i\th:%i\tex:%i\n",ex->start_pc,ex->end_pc,ex->handler,*ex->index);
	}
	
}
