#include <stdio.h>
#include <stdlib.h>
#include <vm/jclass.h>
#include <vm/attributes.h>

FIELD_PTR field_new(POOL_PTR pool,uint16 flag,const char* name,const char* signature)
{
	FIELD_PTR field=malloc(sizeof(FIELD));
	field->pool=pool;
	field->access_flags=flag;
	field->name_index=add_rec_utf8_entry(pool,name);
	field->descriptor_index=add_rec_utf8_entry(pool,signature);
	field->listAttributes= list_create(LISTCOUNT_T_MAX);	
	return field;
}


ATTR_PTR field_add_attribute(FIELD_PTR field, const char* type)
{
	ATTR_PTR attr=malloc(sizeof(ATTR));
	attr->pool=field->pool;
	attr->name_index=add_rec_utf8_entry(field->pool,type);
	list_append(field->listAttributes,lnode_create(attr));
	if(!strcmp(type,CODE))
	{
		attr->tag=CODE_ID;
		attr->info.code=malloc(sizeof(ATTR_CODE));
		attr->info.code->listCode=list_create(LISTCOUNT_T_MAX);
		attr->info.code->hashLabel=hash_create(HASHCOUNT_T_MAX,0,0);
		attr->info.code->listException=list_create(LISTCOUNT_T_MAX);
		attr->info.code->current_pc=0;
	} else if(!strcmp(type,SYNTHETIC))
	{
		attr->tag=SYNTH_ID;
		attr->attr_length=0;
	} else if(!strcmp(type,EXCEPTIONS))
	{
		attr->tag=EXCEP_ID;
		attr->attr_length=0; //To change latter
		attr->info.exception=malloc(sizeof(ATTR_EXCEPTION));
		attr->info.exception->nb=0;
		attr->info.exception->tab=NULL;
	} else if(!strcmp(type,CONSTANT_VALUE))
	{
		attr->tag=CONST_ID;
		attr->attr_length=2; 
		attr->info.constant=malloc(sizeof(ATTR_CONSTANT));
	}
	return attr;
}


/**
* 	it creates the required entry to store the field in the JCLASS structure 
*	and the indexes to the constant pool are fixed because they are obtained (along with the definition
*	of the field) from the parsing of a file
*/
FIELD_PTR field_new_with_fixed_index(POOL_PTR pool, uint16 flag, uint16 name_index, uint16 descriptor_index)
{
	FIELD_PTR field=malloc(sizeof(FIELD));
	field->pool=pool;
	field->access_flags=flag;

	field->name_index = (uint16*)malloc(sizeof(uint16));
	memcpy(field->name_index, &name_index, sizeof(uint16));

	field->descriptor_index = (uint16*)malloc(sizeof(uint16));
	memcpy(field->descriptor_index, &descriptor_index, sizeof(uint16));

	field->listAttributes= list_create(LISTCOUNT_T_MAX);	
	return field;
}

ATTR_PTR field_get_attribute(FIELD_PTR field, const char* type)
{
	lnode_t* node;
	list_t* list = field->listAttributes;	

	ATAG code;
	
	if(!strcmp(type,CODE))
		code = CODE_ID;
	else if(!strcmp(type,SYNTHETIC))
		code = SYNTH_ID;
	else if(!strcmp(type,EXCEPTIONS))
		code = EXCEP_ID;
	else if(!strcmp(type,CONSTANT_VALUE))
		code = CONST_ID;
	
	for(node = list_first(list); node != 0; node = list_next(list, node))
	{
		ATTR_PTR attr = (ATTR_PTR)lnode_get(node);

		if (attr->tag == code)
			return attr;
	}
	return NULL;
}
	
	

