#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/time.h>
#include <string.h> //for strerror
#include <fcntl.h>
//#include <kazlib/list.h>
#include <vm/jclass.h>
#include <vm/jmethods.h>

#include <vm/opcode.h>

#include <io/io.h>



#include <nanohttp/nanohttp-client.h> // for byte_t

#include <vm/constantpoolinfo.h>

#include "event/servicedef.h"

/**
*   Can be called by code extern to this file.
* 	it is used to add a reference to an interface in the definition of a JCLASS
 X pool entry into the constant pool.
* 	it creates all the required entry to store the value in the pool
*	and its index is fixed because it is obtained (along with all the definition
*	of the entry) from the parsing of a file
*/
FIELD_PTR class_add_field_with_fixed_index(JCLASS_PTR jclass, uint16 flag, uint16 name_index, uint16 descriptor_index)
{
	FIELD_PTR field;
	field = field_new_with_fixed_index(jclass->pool, flag, name_index, descriptor_index);	
	list_append(jclass->listFields,lnode_create(field));	
	return field;
}


/**
*   Can be called by code extern to this file.
* 	it is used to add a reference to an interface in the definition of a JCLASS
 X pool entry into the constant pool.
* 	it creates all the required entry to store the value in the pool
*	and its index is fixed because it is obtained (along with all the definition
*	of the entry) from the parsing of a file

void class_add_interface_with_fixed_index(JCLASS_PTR cls, const char* interfaceName)
{
	cls->nbInterfaces+=1;		
	cls->tabInterface=realloc(cls->tabInterface,sizeof(uint16*)*(cls->nbInterfaces));
	uint16* index = add_rec_class_entry(cls->pool,interfaceName);
	cls->tabInterface[cls->nbInterfaces-1]=index;
}
*/

/**
*   Can be called by code extern to this file.
* 	it is used to add a reference to an interface in the definition of a JCLASS
 X pool entry into the constant pool.
* 	it creates all the required entry to store the value in the pool
*	and its index is fixed because it is obtained (along with all the definition
*	of the entry) from the parsing of a file
*/
FIELD_PTR class_add_method_with_fixed_index(JCLASS_PTR jclass, uint16 flag, uint16 name_index, uint16 descriptor_index)
{
	FIELD_PTR field;
	field = field_new_with_fixed_index(jclass->pool, flag, name_index, descriptor_index);	
	list_append(jclass->listMethods,lnode_create(field));	
	return field;
}

void parse_poolEntry_charPtr(char** stream, POOL_PTR pool, int* i)
{
	POOL_ENTRY_PTR entry;
	j_byte tag;
	parsejByte_charPtr(stream, &tag);

	switch(tag)
	{
		case CONSTANT_Utf8:
		{
			char* const_utf8 = parsejUTF8String_charPtr(stream);	
			add_utf8_entry_with_fixed_index(pool, const_utf8, i);
			break;			
		}
		case CONSTANT_Class:
		{
			j_short name_index;
			parsejShort_charPtr(stream, &name_index);
			add_class_entry_with_fixed_index(pool, name_index, i);
			break;
		}
		case CONSTANT_Double:
		{
			fprintf(stdout,"write(Double) NOT IMPLEMENTED YET\n");
			break;
		}
		case CONSTANT_Integer:
		{
			fprintf(stdout,"write(Integer) NOT IMPLEMENTED YET\n");
			break;
		}
		case CONSTANT_Fieldref:
		case CONSTANT_InterfaceMethodref:
		case CONSTANT_Methodref:
		{
			j_short class_index, name_type_index;
			parsejShort_charPtr(stream, &class_index);
			parsejShort_charPtr(stream, &name_type_index);
			add_ref_entry_with_fixed_index(pool, tag, class_index, name_type_index, i);
			break;
		}
		case CONSTANT_NameAndType:
		{
			j_short name_index, descriptor_index;
			parsejShort_charPtr(stream, &name_index);
			parsejShort_charPtr(stream, &descriptor_index);
			add_name_type_entry_with_fixed_index(pool, name_index, descriptor_index, i);
			break;
		}
		case CONSTANT_Long:
		{
			// Not correct yet: I must read two ints and rebuild the long value instead
			// of reading the 8 bytes into a long value

			j_long long_bytes;
			parsejLong_charPtr(stream, &long_bytes);
			add_long_entry_with_fixed_index(pool, long_bytes, i);
			break;
		}
		case CONSTANT_String:
		{
			j_short string_index;
			parsejShort_charPtr(stream, &string_index);
			add_string_entry_with_fixed_index(pool, string_index, i);

			break;
		}
		default:
		{
			fprintf(stdout,"write(Tag:%02X hex) NOT FOUND\n", tag);
			break;			
		}
	}

	
}


void parseConstantPool_charPtr(char** stream, POOL_PTR poolPtr)
{
	int i=0;
	j_short poolsize;

	parsejShort_charPtr(stream , &poolsize);

	fprintf(stdout,"Poolsize = %02x \n", poolsize);

	if(poolsize!=0){
		for(i=1;i<poolsize;i++)
		{
			//POOL_ENTRY_PTR entry;
			fprintf(stdout,"%02x ) ", i);
			parse_poolEntry_charPtr(stream, poolPtr, &i);
			//add_pool_entry(POOL_PTR pool,TAG tag,POOL_ENTRY_PTR entry);

		//	pool->tab[i]->write(fd,pool->tab[i]);	
		}
	}
}

void parseInterfaces_charPtr(char** stream, JCLASS_PTR jclass)
{
	int i;
	
	parsejShort_charPtr(stream, &jclass->nbInterfaces);
	fprintf(stdout,"nb_interfaces = %04X hex\n", jclass->nbInterfaces);
	
	jclass->tabInterface = malloc(sizeof(uint16*)*(jclass->nbInterfaces));

	for(i=0; i < jclass->nbInterfaces; i++)
	{	
		j_short* interface_index = (j_short*)malloc(sizeof(j_short));
		parsejShort_charPtr(stream, interface_index);
		
		jclass->tabInterface[i] = interface_index;
		
		fprintf(stdout,"interface_index = %04X hex\n", *jclass->tabInterface[i]);
	}
}


void parseFields_charPtr(char** stream, JCLASS_PTR jclass)
{
	int i,j,k;
	j_short nb_fields;

	parsejShort_charPtr(stream, &nb_fields);
	fprintf(stdout,"nb_fields = %04X hex\n", nb_fields);

	for(i=0; i < nb_fields; i++)
	{	
		j_short access_flags;
		parsejShort_charPtr(stream, &access_flags);

		j_short name_index;
		parsejShort_charPtr(stream, &name_index);

		j_short descriptor_index;
		parsejShort_charPtr(stream, &descriptor_index);

		j_short attributes_count;
		parsejShort_charPtr(stream, &attributes_count);

		fprintf(stdout,"Field %x) access_flags=%04Xhex name_index=%04Xhex descriptor_index=%04Xhex attributes_count=%04Xhex \n" ,\
			i, access_flags, name_index, descriptor_index, attributes_count);

		for(j=0; j < attributes_count; j++)
		{
			j_short attr_name_index;
			parsejShort_charPtr(stream, &attr_name_index);

			j_int attr_length;
			parsejInt_charPtr(stream, &attr_length);
			
			fprintf(stdout,"\tAttr %x) attr_name_index= %x attr_length=%x", j, attr_name_index, attr_length);

			for(k=0; k < attr_length; k++)
			{
				j_byte attr_info;
				parsejByte_charPtr(stream, &attr_info);
			}
		}
		fprintf(stdout,"\n");

		/*
		 *	skip attributes: attribute parsing is not implemented yet 
		 */
		class_add_field_with_fixed_index(jclass, access_flags, name_index, descriptor_index);

	}

}


void parseMethods_charPtr(char** stream, JCLASS_PTR jclass)
{
	int i,j,k;
	j_short nb_methods;

	parsejShort_charPtr(stream, &nb_methods);
	fprintf(stdout,"nb_methods = %04X hex\n", nb_methods);

	for(i=0; i < nb_methods; i++)
	{	
		j_short access_flags;
		parsejShort_charPtr(stream, &access_flags);

		j_short name_index;
		parsejShort_charPtr(stream, &name_index);

		j_short descriptor_index;
		parsejShort_charPtr(stream, &descriptor_index);

		j_short attributes_count;
		parsejShort_charPtr(stream, &attributes_count);

		fprintf(stdout,"method %x) access_flags=%04Xhex name_index=%04Xhex descriptor_index=%04Xhex attributes_count=%04Xhex \n", i, access_flags, name_index, descriptor_index, attributes_count);

		FIELD_PTR jfield = class_add_method_with_fixed_index(jclass, access_flags, name_index, descriptor_index);

		for(j=0; j < attributes_count; j++)
		{
			ATTR_PTR attr = parse_attr_charPtr(stream, jclass->pool);
			list_append(jfield->listAttributes, lnode_create(attr));
/*
			j_short attr_name_index;
			parsejShort_charPtr(stream, &attr_name_index);
			//stream += 2;

			j_int attr_length;
			parsejInt_charPtr(stream, &attr_length);
			//stream += 4;
			
			fprintf(stdout,"\tAttr %x) attr_name_index= %x attr_length=%x\n", j, attr_name_index, attr_length);

			for(k=0; k < attr_length; k++)
			{
				j_byte attr_info;
				parsejByte_charPtr(stream, &attr_info);
				//stream += 1;

				fprintf(stdout,"%02x ", attr_info&0xff);

			}
*/
		}
		fprintf(stdout,"\n");

		/*
		 *	skip attributes: attribute parsing is not implemented yet 
		 */
		
	}
}


JCLASS_PTR class_parse_charPtr(char* stream)
{

	int i, j, k;
	int nb;

	JCLASS_PTR jclass = new_class0();
	
	j_int magic;
	parsejInt_charPtr(&stream, &magic);

	if (magic != 0xCAFEBABE) {
		fprintf(stdout,"\nError reading magic number %x instead of %x\n", magic, 0xCAFEBABE);  
	} else {
		fprintf(stdout,"\nMagic = %x \n", magic);
	}

	//fflush(stdout);


	parsejShort_charPtr(&stream, &jclass->minor);

	fprintf(stdout,"Minor = %x \n", jclass->minor);

	parsejShort_charPtr(&stream, &jclass->major);

	fprintf(stdout,"Major = %x \n", jclass->major);

	//fflush(stdout);


	//POOL_PTR poolPtr = new_pool();
	parseConstantPool_charPtr(&stream, jclass->pool);
	


	parsejShort_charPtr(&stream, &jclass->modifier);
	fprintf(stdout,"access flag = %04X hex\n", jclass->modifier);

	parsejShort_charPtr(&stream, &jclass->indexClassName);
	fprintf(stdout,"class_index = %04X hex\n", jclass->indexClassName);

	parsejShort_charPtr(&stream, &jclass->indexSuperClassName);
	fprintf(stdout,"superclass_index = %04X hex\n", jclass->indexSuperClassName);

	parseInterfaces_charPtr(&stream, jclass);
	
	parseFields_charPtr(&stream, jclass);


	parseMethods_charPtr(&stream, jclass);

j_byte attrs;
for (i=1; i<13; i++) {
	parsejByte_charPtr(&stream, &attrs);
	fprintf(stdout,"Parsing class attribs=%x",attrs);
}

//	fflush(stdout);


	for (i=1; i<13; i++)
		print_pool(jclass->pool, i);

	return jclass;
}

void get_method_arg_names(FIELD_PTR meth, JMETHOD_PTR meth_tmp)
{
	lnode_t* node;
	list_t* list_attr = meth->listAttributes;

	for(node = list_first(list_attr); node != 0; node = list_next(list_attr, node))
	{
		ATTR_PTR attr = (ATTR_PTR)lnode_get(node);
		fprintf(stdout, "\nFIELD/ATTR begin=: %d . node address=%d\n",attr->tag, node);

		// if the attribute of the method is of Type CODE, it contains
		// the attribute LOCAL_VAR_TABLE (only if the class has been compiled with -g option for debug)
		if (attr->tag == CODE_ID) {

			fprintf(stdout, "\nCODEID begin\n");

			list_t* code_attr_list = attr->info.code->listAttributes;
			lnode_t* code_attr_list_node;
	
			for(code_attr_list_node = list_first(code_attr_list); code_attr_list_node != 0; 
					code_attr_list_node = list_next(code_attr_list, code_attr_list_node)) {
		           ATTR_PTR code_attr = (ATTR_PTR)lnode_get(code_attr_list_node);
		
			   fprintf(stdout, "\nCODE/ATTR begin=: %d\n",code_attr->tag);

			   if (code_attr->tag == LOCALVAR_ID) {
				list_t* local_vars = code_attr->info.local_var_table->local_var_table;
 
				fprintf(stdout, "\nNumber of arguments: %d\n",list_count(local_vars));

				lnode_t* var_node;
				int index_node = 0;
				for(var_node = list_first(local_vars); var_node != 0; var_node = list_next(local_vars, var_node))
				{

					fprintf(stdout, "\nCODE/LOCALVAR/ATTR begin\n");

					LOCAL_VAR_ENTRY_PTR var = (LOCAL_VAR_ENTRY_PTR)lnode_get(var_node);
					POOL_ENTRY_PTR* tab = var->pool->tab;

					char* name = utf8ToString(tab[*var->name_index]->info.utf8->utf8);
					char* type = utf8ToString(tab[*var->descriptor_index]->info.utf8->utf8);
					fprintf(stdout, "\n\targument name: %d, descriptor: %d ",*var->name_index, *var->descriptor_index);
					fprintf(stdout, "\n\targument name: %s, type:%s\n", name, type);

					fprintf(stdout, "\nCODE/LOCALVAR/ATTR end\n");
					// Add argument to meth_tmp: skip first argument because
					// it is always the "this" value referring to the object itself
					if (index_node > 0)
						JMethod_add_arg(meth_tmp, name, type);
					index_node++;
				}
			   }
			   fprintf(stdout, "\nCODE/ATTR end=: %d\n",code_attr->tag);
   
			}
			fprintf(stdout, "\nCODEID end\n");

		}
		fprintf(stdout, "\nFIELD/ATTR end=: %d\n",attr->tag);
	}
}

void get_methods(JCLASS_PTR jclass, uint16 access_flags, JMETHODS_PTR jmeths/*char* methNames[], char* methSignatures[], int* nMeth*/)
{
	lnode_t* node;
	list_t* list;	

	int nMeth = 0;


	//methNames = NULL;
	//methSignatures = NULL;

	list = jclass->listMethods;
	fprintf(stdout, "number of methods: %d\n",list_count(list));
	int index_node = 0;

	for(node = list_first(list); node != 0; node = list_next(list, node))
	{

		/*
		 * skip first method in the list because it is 
		 * always the constructor <init> method
		 */
		if (index_node == 0) {
			index_node++;	
			continue;
		}

		FIELD_PTR meth = (FIELD_PTR)lnode_get(node);

		if (meth->access_flags & access_flags) {

		   (nMeth)++;
		   fprintf(stdout, "\nMethod %d: name = %x, signature=%x FLAGS=%04x",nMeth-1, *meth->name_index, *meth->descriptor_index, meth->access_flags);

		   POOL_ENTRY_PTR* tab = meth->pool->tab;

		   char* name = utf8ToString(tab[*meth->name_index]->info.utf8->utf8);
		   char* signature = utf8ToString(tab[*meth->descriptor_index]->info.utf8->utf8);


//	char**	methNames;
//	char** 	methSignatures;

/*
typedef struct JMethod
{
	char*	methName;
	char* 	methSignatures;
	int 	nArgs;
	char**	args;
} JMETHOD;

typedef struct JMethods
{
	list_t*	methods;
	int 	nMeths;
} JMETHODS;
*/

		   JMETHOD_PTR meth_tmp = create_JMethod_2(name, signature);
		   get_method_arg_names(meth, meth_tmp);

		   fprintf(stdout, "\n\tMethod: name = %s, signature=%s", meth_tmp->methName,  meth_tmp->methSignature);

		   list_append(jmeths->methods, lnode_create(meth_tmp));
		   
		   //jmeths->methNames = (char**)realloc(jmeths->methNames, sizeof(char*)*(nMeth));
		   //jmeths->methSignatures = (char**)realloc(jmeths->methSignatures, sizeof(char*)*(nMeth));

		   //jmeths->methNames[nMeth-1] = name;
		  // jmeths->methSignatures[nMeth-1] = signature;

		   //fprintf(stdout, "\n\tMethod: name = %s, signature=%s", jmeths->methNames[nMeth-1],  jmeths->methSignatures[nMeth-1]);
		   //get_method_arg_names(meth);


/*
fprintf(stdout, "\n\tALLOCATION methName before =%d",methNames);
		   methNames = (char**)realloc(methNames, sizeof(char*)*(*nMeth));
fprintf(stdout, "\n\tALLOCATION methName after =%d",methNames);
		   methNames[*nMeth-1] = name;
fprintf(stdout, "\n\tALLOCATION methSign before =%d",methSignatures);
		   methSignatures = (char**)realloc(methSignatures, sizeof(char*)*(*nMeth));
fprintf(stdout, "\n\tALLOCATION methSign after =%d",methSignatures);
		   methSignatures[*nMeth-1] = signature;
*/
		   
/*
		   *meths = (METHODDEFPTR)realloc(*meths, sizeof(METHODDEF)*(*nMeth));
		   (*meths)[*nMeth] = methoddef_new();
		   methoddef_set_name((*meths)[*nMeth],  name);
*/


		}
	}
	jmeths->nMeths = nMeth;
}

void get_public_methods(JCLASS_PTR jclass, JMETHODS_PTR meths/*char* methNames[], char* methSignatures[],  int* nMeth */)
{
	get_methods(jclass, ACC_PUBLIC, meths /*methNames, methSignatures, nMeth*/);
}

JMETHODS_PTR create_JMethods()
{
	JMETHODS_PTR meths = (JMETHODS_PTR)malloc(sizeof(JMETHODS));
	meths->methods = list_create(LISTCOUNT_T_MAX);
	meths->nMeths = 0;
	return meths;
}


JMETHOD_PTR create_JMethod()
{
	JMETHOD_PTR meth = (JMETHOD_PTR)malloc(sizeof(JMETHOD));
	meth->methName = NULL;
	meth->methSignature = NULL;
	meth->nArgs = 0;
	meth->args_names = NULL;
	meth->args_types = NULL;
	return meth;
}

JMETHOD_PTR create_JMethod_2(char* name, char* signature)
{
	JMETHOD_PTR meth = (JMETHOD_PTR)malloc(sizeof(JMETHOD));
	meth->methName = malloc((strlen(name)+1)*sizeof(char));
	memcpy(meth->methName, name, strlen(name)+1);
	meth->methSignature = malloc((strlen(signature)+1)*sizeof(char));
	memcpy(meth->methSignature, signature, strlen(signature)+1);
	meth->nArgs = 0;
	meth->args_names = NULL;
	meth->args_types = NULL;
	return meth;
}

int JMethod_add_arg(JMETHOD_PTR meth, char* name, char* type)
{
	meth->args_names = (char**)realloc(meth->args_names, sizeof(char*)*(meth->nArgs));
	meth->args_names[meth->nArgs] = name;
	meth->args_types = (char**)realloc(meth->args_types, sizeof(char*)*(meth->nArgs));
	meth->args_types[meth->nArgs] = type;
	meth->nArgs++;
	return meth->nArgs;
}

char* class_get_classname(JCLASS_PTR jclass)
{
	uint16* index_class_name = jclass->indexClassName;
	POOL_ENTRY_PTR* tab = jclass->pool->tab;	
	
	uint16* index_class_name_utf8 =  tab[(int)jclass->indexClassName]->info.cl->name_index;
	char* class_name = utf8ToString(tab[*index_class_name_utf8]->info.utf8->utf8);
	                                    
	return class_name;
}

